@@ -23,14 +23,18 @@
 // * acceptance of all terms of the Geant4 Software license.          *
 // ********************************************************************
 // 
-// class G4LogicalVolume implementation
+// G4LogicalVolume implementation
 //
-// 15.01.13 G.Cosmo, A.Dotti: Modified for thread-safety for MT
-// 01.03.05 G.Santin: Added flag for optional propagation of GetMass()
-// 17.05.02 G.Cosmo: Added flag for optional optimisation
-// 12.02.99 S.Giani: Default initialization of voxelization quality
-// 04.08.97 P.M.DeFreitas: Added methods for parameterised simulation 
 // 11.07.95 P.Kent: Initial version
+// 04.08.97 P.M.DeFreitas: Added methods for parameterised simulation 
+// 09.11.98 M.Verderi, J.Apostolakis: Added BiasWeight member and accessors
+// 12.02.99 S.Giani: Added user defined optimisation quality
+// 18.04.01 G.Cosmo: Migrated to STL vector
+// 17.05.02 G.Cosmo: Added IsToOptimise() method and related flag
+// 24.09.02 G.Cosmo: Added flags and accessors for region cuts handling
+// 12.11.04 G.Cosmo: Added GetMass() method for computing mass of the tree
+// 01.03.05 G.Santin: Added flag for optional propagation of GetMass()
+// 15.01.13 G.Cosmo, A.Dotti: Modified for thread-safety for MT
 // --------------------------------------------------------------------
 
 #include "G4LogicalVolume.hh"
@@ -512,10 +516,10 @@ G4LogicalVolume::IsAncestor(const G4VPhysicalVolume* aVolume) const
   G4bool isDaughter = IsDaughter(aVolume);
   if (!isDaughter)
   {
-    for (auto itDau = fDaughters.cbegin(); itDau != fDaughters.cend(); ++itDau)
+    for (const auto & daughter : fDaughters)
     {
-      isDaughter = (*itDau)->GetLogicalVolume()->IsAncestor(aVolume);
-      if (isDaughter)  break;
+      isDaughter = daughter->GetLogicalVolume()->IsAncestor(aVolume);
+      if (isDaughter) { break; }
     }
   }
   return isDaughter;
@@ -531,9 +535,8 @@ G4LogicalVolume::IsAncestor(const G4VPhysicalVolume* aVolume) const
 G4int G4LogicalVolume::TotalVolumeEntities() const
 {
   G4int vols = 1;
-  for (auto itDau = fDaughters.cbegin(); itDau != fDaughters.cend(); ++itDau)
+  for (auto physDaughter : fDaughters)
   {
-    G4VPhysicalVolume* physDaughter = (*itDau);
     vols += physDaughter->GetMultiplicity()
            *physDaughter->GetLogicalVolume()->TotalVolumeEntities();
   }
@@ -599,9 +602,8 @@ G4double G4LogicalVolume::GetMass(G4bool forced,
   // and if required by the propagate flag, add the real daughter's
   // one computed recursively
 
-  for (auto itDau = fDaughters.cbegin(); itDau != fDaughters.cend(); ++itDau)
+  for (const auto & physDaughter : fDaughters)
   {
-    G4VPhysicalVolume* physDaughter = (*itDau);
     G4LogicalVolume* logDaughter = physDaughter->GetLogicalVolume();
     G4double subMass = 0.0;
     G4VSolid* daughterSolid = nullptr;
@@ -679,7 +681,7 @@ G4bool G4LogicalVolume::ChangeDaughtersType(EVolume aType)
 //
 void G4LogicalVolume::SetVisAttributes (const G4VisAttributes& VA)
 {
-  if (G4Threading::IsWorkerThread()) return;
+  if (G4Threading::IsWorkerThread()) { return; }
   fVisAttributes = std::make_shared<const G4VisAttributes>(VA);
 }
 
@@ -689,6 +691,6 @@ void G4LogicalVolume::SetVisAttributes (const G4VisAttributes& VA)
 //
 void G4LogicalVolume::SetVisAttributes (const G4VisAttributes* pVA)
 {
-  if (G4Threading::IsWorkerThread()) return;
+  if (G4Threading::IsWorkerThread()) { return; }
   fVisAttributes = std::shared_ptr<const G4VisAttributes>(pVA,[](const G4VisAttributes*){});
 }
