@@ -25,15 +25,13 @@
 // 
 // class G4MultiNavigator Implementation
 //
-// Author:  John Apostolakis, November 2006
+// Author: John Apostolakis (CERN), November 2006
 // --------------------------------------------------------------------
 
 #include <iomanip>
 
 #include "G4MultiNavigator.hh"
 
-class G4FieldManager;
-
 #include "G4SystemOfUnits.hh"
 #include "G4Navigator.hh"
 #include "G4PropagatorInField.hh"
@@ -73,10 +71,6 @@ G4MultiNavigator::G4MultiNavigator()
 
 // -----------------------------------------------------------------------
 
-G4MultiNavigator::~G4MultiNavigator() = default;
-
-// -----------------------------------------------------------------------
-
 G4double G4MultiNavigator::ComputeStep(const G4ThreeVector& pGlobalPoint,
                                        const G4ThreeVector& pDirection,
                                        const G4double       proposedStepLength,
