@@ -48,12 +48,16 @@
 
 #include "G4VPVParameterisation.hh"
 
-#include "meshdefs.hh"
-
-#include "Randomize.hh"
+#include "G4QuickRand.hh"
 
 #include "G4VGraphicsScene.hh"
 #include "G4Polyhedron.hh"
+#include "G4AutoLock.hh"
+
+namespace
+{
+  G4Mutex torusMutex = G4MUTEX_INITIALIZER;
+}
 
 using namespace CLHEP;
 
@@ -177,18 +181,6 @@ G4Torus::G4Torus( __void__& a )
 {
 }
 
-//////////////////////////////////////////////////////////////////////
-//
-// Destructor
-
-G4Torus::~G4Torus() = default;
-
-//////////////////////////////////////////////////////////////////////////
-//
-// Copy constructor
-
-G4Torus::G4Torus(const G4Torus&) = default;
-
 //////////////////////////////////////////////////////////////////////////
 //
 // Assignment operator
@@ -487,28 +479,28 @@ G4bool G4Torus::CalculateExtent( const EAxis pAxis,
 
   // define vectors for bounding envelope
   G4ThreeVectorList pols[NDISK+1];
-  for (auto & pol : pols) pol.resize(4);
+  for (auto & pol : pols) { pol.resize(4); }
 
   std::vector<const G4ThreeVectorList *> polygons;
   polygons.resize(NDISK+1);
-  for (G4int k=0; k<NDISK+1; ++k) polygons[k] = &pols[k];
+  for (G4int k=0; k<NDISK+1; ++k) { polygons[k] = &pols[k]; }
 
   // set internal and external reference circles
   G4TwoVector rzmin[NDISK];
   G4TwoVector rzmax[NDISK];
 
-  if ((rtor-rmin*sinHalfDisk)/cosHalf > (rtor+rmin*sinHalfDisk)) rmin = 0;
+  if ((rtor-rmin*sinHalfDisk)/cosHalf > (rtor+rmin*sinHalfDisk)) { rmin = 0; }
   rmax /= cosHalfDisk;
   G4double sinCurDisk = sinHalfDisk;
   G4double cosCurDisk = cosHalfDisk;
   for (G4int k=0; k<NDISK; ++k)
   {
     G4double rmincur = rtor + rmin*cosCurDisk;
-    if (cosCurDisk < 0 && rmin > 0) rmincur /= cosHalf;
+    if (cosCurDisk < 0 && rmin > 0) { rmincur /= cosHalf; }
     rzmin[k].set(rmincur,rmin*sinCurDisk);
 
     G4double rmaxcur = rtor + rmax*cosCurDisk;
-    if (cosCurDisk > 0) rmaxcur /= cosHalf;
+    if (cosCurDisk > 0) { rmaxcur /= cosHalf; }
     rzmax[k].set(rmaxcur,rmax*sinCurDisk);
 
     G4double sinTmpDisk = sinCurDisk;
@@ -560,10 +552,13 @@ G4bool G4Torus::CalculateExtent( const EAxis pAxis,
     // set bounding envelope for current slice and adjust extent
     G4double emin,emax;
     G4BoundingEnvelope benv(bmin,bmax,polygons);
-    if (!benv.CalculateExtent(pAxis,pVoxelLimit,pTransform,emin,emax)) continue;
-    if (emin < pMin) pMin = emin;
-    if (emax > pMax) pMax = emax;
-    if (eminlim > pMin && emaxlim < pMax) break; // max possible extent
+    if (!benv.CalculateExtent(pAxis,pVoxelLimit,pTransform,emin,emax))
+    {
+      continue;
+    }
+    if (emin < pMin) { pMin = emin; }
+    if (emax > pMax) { pMax = emax; }
+    if (eminlim > pMin && emaxlim < pMax) { break; } // max possible extent
   }
   return (pMin < pMax);
 }
@@ -583,8 +578,7 @@ EInside G4Torus::Inside( const G4ThreeVector& p ) const
   r   = std::hypot(p.x(),p.y());
   pt2 = p.z()*p.z() + (r-fRtor)*(r-fRtor);
 
-  if (fRmin != 0.0) tolRMin = fRmin + fRminTolerance ;
-  else       tolRMin = 0 ;
+  (fRmin != 0.0) ? (tolRMin = fRmin + fRminTolerance) : (tolRMin = 0);
 
   tolRMax = fRmax - fRmaxTolerance;
 
@@ -703,7 +697,7 @@ G4ThreeVector G4Torus::SurfaceNormal( const G4ThreeVector& p ) const
   pt  = std::hypot(p.z(),rho-fRtor);
 
   G4double  distRMax = std::fabs(pt - fRmax);
-  if(fRmin != 0.0) distRMin = std::fabs(pt - fRmin);
+  if(fRmin != 0.0) { distRMin = std::fabs(pt - fRmin); }
 
   if( rho > delta && pt != 0.0 )
   {
@@ -863,7 +857,7 @@ G4ThreeVector G4Torus::ApproxSurfaceNormal( const G4ThreeVector& p ) const
 
     if (distSPhi < distEPhi) // Find new minimum
     {
-      if (distSPhi<distMin) side = kNSPhi ;
+      if (distSPhi<distMin) { side = kNSPhi ; }
     }
     else
     {
@@ -936,9 +930,9 @@ G4double G4Torus::DistanceToIn( const G4ThreeVector& p,
   G4double distX = std::abs(p.x()) - boxDx;
   G4double distY = std::abs(p.y()) - boxDy;
   G4double distZ = std::abs(p.z()) - boxDz;
-  if (distX >= -halfCarTolerance && p.x()*v.x() >= 0) return kInfinity;
-  if (distY >= -halfCarTolerance && p.y()*v.y() >= 0) return kInfinity;
-  if (distZ >= -halfCarTolerance && p.z()*v.z() >= 0) return kInfinity;
+  if (distX >= -halfCarTolerance && p.x()*v.x() >= 0) { return kInfinity; }
+  if (distY >= -halfCarTolerance && p.y()*v.y() >= 0) { return kInfinity; }
+  if (distZ >= -halfCarTolerance && p.z()*v.z() >= 0) { return kInfinity; }
 
   // Calculate safety distance to bounding box
   // If point is too far, move it closer and calculate distance
@@ -1593,43 +1587,71 @@ std::ostream& G4Torus::StreamInfo( std::ostream& os ) const
 
 G4ThreeVector G4Torus::GetPointOnSurface() const
 {
-  G4double cosu, sinu,cosv, sinv, aOut, aIn, aSide, chose, phi, theta, rRand;
+  G4double rrmin = fRmin*fRmin;
+  G4double rrmax = fRmax*fRmax;
+  G4double smax = twopi*fRtor*fDPhi*fRmax;
+  G4double smin = twopi*fRtor*fDPhi*fRmin;
+  G4double sphi = (fDPhi >= twopi) ? 0. : pi*(rrmax - rrmin);
 
-  phi   = G4RandFlat::shoot(fSPhi,fSPhi+fDPhi);
-  theta = G4RandFlat::shoot(0.,twopi);
-
-  cosu   = std::cos(phi);    sinu = std::sin(phi);
-  cosv   = std::cos(theta);  sinv = std::sin(theta); 
-
-  // compute the areas
-
-  aOut   = (fDPhi)*twopi*fRtor*fRmax;
-  aIn    = (fDPhi)*twopi*fRtor*fRmin;
-  aSide  = pi*(fRmax*fRmax-fRmin*fRmin);
-
-  if ((fSPhi == 0) && (fDPhi == twopi)){ aSide = 0; }
-  chose = G4RandFlat::shoot(0.,aOut + aIn + 2.*aSide);
-
-  if(chose < aOut)
+  G4double u = G4QuickRand();
+  G4double v = twopi*G4QuickRand();;
+  G4double select = (smax + smin + 2.*sphi)*G4QuickRand();
+  G4double phi, r, ds;
+  if (select < 2.*sphi)
   {
-    return { (fRtor+fRmax*cosv)*cosu, (fRtor+fRmax*cosv)*sinu, fRmax*sinv };
-  }
-  else if( (chose >= aOut) && (chose < aOut + aIn) )
-  {
-    return { (fRtor+fRmin*cosv)*cosu, (fRtor+fRmin*cosv)*sinu, fRmin*sinv };
-  }
-  else if( (chose >= aOut + aIn) && (chose < aOut + aIn + aSide) )
-  {
-    rRand = GetRadiusInRing(fRmin,fRmax);
-    return { (fRtor+rRand*cosv)*std::cos(fSPhi),
-             (fRtor+rRand*cosv)*std::sin(fSPhi), rRand*sinv };
+    // phi cut
+    phi = fSPhi + fDPhi*(G4double)(select < sphi);
+    r = std::sqrt(rrmin + (rrmax - rrmin)*u);
+    ds = fRtor + r*std::cos(v);
   }
   else
-  {   
-    rRand = GetRadiusInRing(fRmin,fRmax);
-    return { (fRtor+rRand*cosv)*std::cos(fSPhi+fDPhi),
-             (fRtor+rRand*cosv)*std::sin(fSPhi+fDPhi), rRand*sinv };
+  {
+    // toroidal surface (rejection sampling)
+    phi = fSPhi + fDPhi*u;
+    r = (select < 2.*sphi + smax) ? fRmax : fRmin;
+    ds = fRtor + r*std::cos(v);
+    for (auto i = 0; i < 10; ++i)
+    {
+      if ((fRtor + r)*G4QuickRand() < ds) { break; }
+      v = twopi*G4QuickRand();
+      ds = fRtor + r*std::cos(v);
+    }
   }
+  return { ds*std::cos(phi), ds*std::sin(phi), r*std::sin(v) };
+}
+
+////////////////////////////////////////////////////////////////////////////
+//
+// GetCubicVolume
+
+G4double G4Torus::GetCubicVolume()
+{
+  if (fCubicVolume == 0)
+  {
+    G4AutoLock l(&torusMutex);
+    fCubicVolume = fDPhi*CLHEP::pi*fRtor*(fRmax*fRmax-fRmin*fRmin);
+    l.unlock();
+  }
+  return fCubicVolume;
+}
+
+////////////////////////////////////////////////////////////////////////////
+//
+// GetSurfaceArea
+
+G4double G4Torus::GetSurfaceArea()
+{
+  if (fSurfaceArea == 0)
+  { 
+    G4AutoLock l(&torusMutex);
+    fSurfaceArea = fDPhi*CLHEP::twopi*fRtor*(fRmax+fRmin);
+    if(fDPhi < CLHEP::twopi)
+    {
+      fSurfaceArea = fSurfaceArea + CLHEP::twopi*(fRmax*fRmax-fRmin*fRmin);
+    } 
+    l.unlock();
+  }
+  return fSurfaceArea;
 }
 
 ///////////////////////////////////////////////////////////////////////
