@@ -37,21 +37,13 @@
 
 #if !(defined(G4GEOM_USE_UELLIPTICALCONE) && defined(G4GEOM_USE_SYS_USOLIDS))
 
-#include "globals.hh"
-
 #include "G4EllipticalCone.hh"
 
-#include "G4RandomTools.hh"
 #include "G4GeomTools.hh"
-#include "G4ClippablePolygon.hh"
 #include "G4VoxelLimits.hh"
 #include "G4AffineTransform.hh"
 #include "G4BoundingEnvelope.hh"
-#include "G4GeometryTolerance.hh"
-
-#include "meshdefs.hh"
-
-#include "Randomize.hh"
+#include "G4QuickRand.hh"
 
 #include "G4VGraphicsScene.hh"
 #include "G4VisExtent.hh"
@@ -61,6 +53,7 @@
 namespace
 {
   G4Mutex polyhedronMutex = G4MUTEX_INITIALIZER;
+  G4Mutex elconeMutex = G4MUTEX_INITIALIZER;
 }
 
 using namespace CLHEP;
@@ -101,6 +94,7 @@ G4EllipticalCone::G4EllipticalCone(const G4String& pName,
 
   SetSemiAxis( pxSemiAxis, pySemiAxis, pzMax );
   SetZCut(pzTopCut);
+  GetSurfaceArea();
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -131,6 +125,7 @@ G4EllipticalCone::~G4EllipticalCone()
 G4EllipticalCone::G4EllipticalCone(const G4EllipticalCone& rhs)
   : G4VSolid(rhs), halfCarTol(rhs.halfCarTol),
     fCubicVolume(rhs.fCubicVolume), fSurfaceArea(rhs.fSurfaceArea),
+    fMinZBaseArea(rhs.fMinZBaseArea), fMaxZBaseArea(rhs.fMaxZBaseArea),
     xSemiAxis(rhs.xSemiAxis), ySemiAxis(rhs.ySemiAxis),
     zheight(rhs.zheight), zTopCut(rhs.zTopCut),
     cosAxisMin(rhs.cosAxisMin), invXX(rhs.invXX), invYY(rhs.invYY)
@@ -141,7 +136,7 @@ G4EllipticalCone::G4EllipticalCone(const G4EllipticalCone& rhs)
 //
 // Assignment operator
 
-G4EllipticalCone& G4EllipticalCone::operator = (const G4EllipticalCone& rhs) 
+G4EllipticalCone& G4EllipticalCone::operator = (const G4EllipticalCone& rhs)
 {
    // Check assignment to self
    //
@@ -155,6 +150,7 @@ G4EllipticalCone& G4EllipticalCone::operator = (const G4EllipticalCone& rhs)
    //
    halfCarTol = rhs.halfCarTol;
    fCubicVolume = rhs.fCubicVolume; fSurfaceArea = rhs.fSurfaceArea;
+   fMinZBaseArea = rhs.fMinZBaseArea; fMaxZBaseArea = rhs.fMaxZBaseArea;
    xSemiAxis = rhs.xSemiAxis; ySemiAxis = rhs.ySemiAxis;
    zheight = rhs.zheight; zTopCut = rhs.zTopCut;
    cosAxisMin = rhs.cosAxisMin; invXX = rhs.invXX; invYY = rhs.invYY;
@@ -165,6 +161,41 @@ G4EllipticalCone& G4EllipticalCone::operator = (const G4EllipticalCone& rhs)
    return *this;
 }
 
+/////////////////////////////////////////////////////////////////////////
+//
+// Set new semi axis
+
+void G4EllipticalCone::SetSemiAxis (G4double newxSemiAxis,
+                                    G4double newySemiAxis,
+                                    G4double newzMax)
+{
+  xSemiAxis = newxSemiAxis;
+  ySemiAxis = newySemiAxis;
+  zheight   = newzMax;
+  if (zTopCut > zheight) { zTopCut = zheight; }
+  G4double axmin = std::min(xSemiAxis, ySemiAxis);
+  cosAxisMin = axmin/std::sqrt(1. + axmin*axmin);
+  invXX = 1./(xSemiAxis*xSemiAxis);
+  invYY = 1./(ySemiAxis*ySemiAxis);
+  fCubicVolume = 0.0;
+  fSurfaceArea = 0.0;
+  GetSurfaceArea();
+  fRebuildPolyhedron = true;
+}
+
+/////////////////////////////////////////////////////////////////////////
+//
+// Set new Z cut
+
+void G4EllipticalCone::SetZCut (G4double newzTopCut)
+{
+  zTopCut = std::min(newzTopCut, zheight);
+  fCubicVolume = 0.0;
+  fSurfaceArea = 0.0;
+  GetSurfaceArea();
+  fRebuildPolyhedron = true;
+}
+
 /////////////////////////////////////////////////////////////////////////
 //
 // Get bounding box
@@ -173,7 +204,7 @@ void G4EllipticalCone::BoundingLimits(G4ThreeVector& pMin,
                                       G4ThreeVector& pMax) const
 {
   G4double zcut   = GetZTopCut();
-  G4double height = GetZMax(); 
+  G4double height = GetZMax();
   G4double xmax   = GetSemiAxisX()*(height+zcut);
   G4double ymax   = GetSemiAxisY()*(height+zcut);
   pMin.set(-xmax,-ymax,-zcut);
@@ -228,7 +259,7 @@ G4EllipticalCone::CalculateExtent(const EAxis pAxis,
   static const G4double sinStep = 2.*sinHalf*cosHalf;
   static const G4double cosStep = 1. - 2.*sinHalf*sinHalf;
   G4double zcut   = bmax.z();
-  G4double height = GetZMax(); 
+  G4double height = GetZMax();
   G4double sxmin  = GetSemiAxisX()*(height-zcut)/cosHalf;
   G4double symin  = GetSemiAxisY()*(height-zcut)/cosHalf;
   G4double sxmax  = bmax.x()/cosHalf;
@@ -241,7 +272,7 @@ G4EllipticalCone::CalculateExtent(const EAxis pAxis,
   {
     baseA[k].set(sxmax*cosCur,symax*sinCur,-zcut);
     baseB[k].set(sxmin*cosCur,symin*sinCur, zcut);
-    
+
     G4double sinTmp = sinCur;
     sinCur = sinCur*cosStep + cosCur*sinStep;
     cosCur = cosCur*cosStep - sinTmp*sinStep;
@@ -266,7 +297,7 @@ EInside G4EllipticalCone::Inside(const G4ThreeVector& p) const
   G4double dz = std::abs(p.z()) - zTopCut;
   G4double dist = std::max(ds,dz);
 
-  if (dist > halfCarTol) return kOutside;
+  if (dist > halfCarTol) { return kOutside; }
   return (dist > -halfCarTol) ? kSurface : kInside;
 }
 
@@ -285,7 +316,7 @@ G4ThreeVector G4EllipticalCone::SurfaceNormal( const G4ThreeVector& p) const
   {
     norm = G4ThreeVector(p.x()*invXX, p.y()*invYY, hp - p.z());
     G4double mag = norm.mag();
-    if (mag == 0) return {0,0,1}; // apex
+    if (mag == 0) { return {0,0,1}; } // apex
     norm *= (1/mag);
     ++nsurf;
   }
@@ -296,28 +327,32 @@ G4ThreeVector G4EllipticalCone::SurfaceNormal( const G4ThreeVector& p) const
     ++nsurf;
   }
 
-  if      (nsurf == 1) return norm;
-  else if (nsurf >  1) return norm.unit(); // elliptic edge
-  else
+  if (nsurf == 1)
   {
-    // Point is not on the surface
-    //
-#ifdef G4CSGDEBUG
-    std::ostringstream message;
-    G4long oldprc = message.precision(16);
-    message << "Point p is not on surface (!?) of solid: "
-            << GetName() << G4endl;
-    message << "Position:\n";
-    message << "   p.x() = " << p.x()/mm << " mm\n";
-    message << "   p.y() = " << p.y()/mm << " mm\n";
-    message << "   p.z() = " << p.z()/mm << " mm";
-    G4cout.precision(oldprc);
-    G4Exception("G4EllipticalCone::SurfaceNormal(p)", "GeomSolids1002",
-                JustWarning, message );
-    DumpInfo();
-#endif
-    return ApproxSurfaceNormal(p);
+    return norm;
   }
+  if (nsurf >  1)
+  {
+    return norm.unit(); // elliptic edge
+  }
+
+  // Point is not on the surface
+  //
+#ifdef G4CSGDEBUG
+  std::ostringstream message;
+  G4long oldprc = message.precision(16);
+  message << "Point p is not on surface (!?) of solid: "
+          << GetName() << G4endl;
+  message << "Position:\n";
+  message << "   p.x() = " << p.x()/mm << " mm\n";
+  message << "   p.y() = " << p.y()/mm << " mm\n";
+  message << "   p.z() = " << p.z()/mm << " mm";
+  G4cout.precision(oldprc);
+  G4Exception("G4EllipticalCone::SurfaceNormal(p)", "GeomSolids1002",
+              JustWarning, message );
+  DumpInfo();
+#endif
+  return ApproxSurfaceNormal(p);
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -333,9 +368,10 @@ G4EllipticalCone::ApproxSurfaceNormal(const G4ThreeVector& p) const
   G4double ds = (hp - zheight)*cosAxisMin;
   G4double dz = std::abs(p.z()) - zTopCut;
   if (ds > dz && std::abs(hp - p.z()) > halfCarTol)
+  {
     return G4ThreeVector(p.x()*invXX, p.y()*invYY, hp - p.z()).unit();
-  else
-    return { 0., 0., (G4double)((p.z() < 0) ? -1. : 1.) };
+  }
+  return { 0., 0., (G4double)((p.z() < 0) ? -1. : 1.) };
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -368,8 +404,8 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
       // As long as we are far enough away, we know we
       // can't intersect
       //
-      if (sigz < 0) return kInfinity;
-      
+      if (sigz < 0) { return kInfinity; }
+
       //
       // Otherwise, we don't intersect unless we are
       // on the surface of the ellipse
@@ -377,8 +413,9 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
 
       if ( sqr(p.x()/( xSemiAxis - halfCarTol ))
          + sqr(p.y()/( ySemiAxis - halfCarTol )) <= sqr( zheight + zTopCut ) )
+      {
         return kInfinity;
-
+      }
     }
     else
     {
@@ -386,13 +423,13 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
       // How far?
       //
       G4double q = -sigz/v.z();
-      
+
       //
       // Where does that place us?
       //
       G4double xi = p.x() + q*v.x(),
                yi = p.y() + q*v.y();
-      
+
       //
       // Is this on the surface (within ellipse)?
       //
@@ -403,8 +440,7 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
         //
         return (sigz < -halfCarTol) ? q : 0;
       }
-      else if (xi/(xSemiAxis*xSemiAxis)*v.x()
-             + yi/(ySemiAxis*ySemiAxis)*v.y() >= 0)
+      if (xi/(xSemiAxis*xSemiAxis)*v.x() + yi/(ySemiAxis*ySemiAxis)*v.y() >= 0)
       {
         //
         // Else, if we are traveling outwards, we know
@@ -419,20 +455,21 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
   // Check z = +dz planer surface
   //
   sigz = p.z() - zTopCut;
-  
+
   if (sigz > -halfCarTol)
   {
     if (v.z() >= 0)
     {
-
-      if (sigz > 0) return kInfinity;
+      if (sigz > 0) { return kInfinity; }
 
       if ( sqr(p.x()/( xSemiAxis - halfCarTol ))
          + sqr(p.y()/( ySemiAxis - halfCarTol )) <= sqr( zheight-zTopCut ) )
+      {
         return kInfinity;
-
+      }
     }
-    else {
+    else
+    {
       G4double q = -sigz/v.z();
 
       G4double xi = p.x() + q*v.x(),
@@ -442,95 +479,88 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
       {
         return (sigz > -halfCarTol) ? q : 0;
       }
-      else if (xi/(xSemiAxis*xSemiAxis)*v.x()
-             + yi/(ySemiAxis*ySemiAxis)*v.y() >= 0)
+      if (xi/(xSemiAxis*xSemiAxis)*v.x() + yi/(ySemiAxis*ySemiAxis)*v.y() >= 0)
       {
-        //        return kInfinity;
+        // return kInfinity;
       }
     }
   }
 
-
 #if 0
 
   // check to see if Z plane is relevant
   //
   if (p.z() < -zTopCut - halfCarTol)
   {
-    if (v.z() <= 0.0)
-      return distMin; 
+    if (v.z() <= 0.0) { return distMin; }
 
     G4double lambda = (-zTopCut - p.z())/v.z();
-    
-    if ( sqr((lambda*v.x()+p.x())/xSemiAxis) + 
+
+    if ( sqr((lambda*v.x()+p.x())/xSemiAxis) +
          sqr((lambda*v.y()+p.y())/ySemiAxis) <=
-         sqr(zTopCut + zheight + halfCarTol) ) 
-    { 
-      return distMin = std::fabs(lambda);    
+         sqr(zTopCut + zheight + halfCarTol) )
+    {
+      return distMin = std::fabs(lambda);
     }
   }
 
-  if (p.z() > zTopCut + halfCarTol) 
+  if (p.z() > zTopCut + halfCarTol)
   {
-    if (v.z() >= 0.0)
-      { return distMin; }
+    if (v.z() >= 0.0) { return distMin; }
 
     G4double lambda  = (zTopCut - p.z()) / v.z();
 
-    if ( sqr((lambda*v.x() + p.x())/xSemiAxis) + 
+    if ( sqr((lambda*v.x() + p.x())/xSemiAxis) +
          sqr((lambda*v.y() + p.y())/ySemiAxis) <=
          sqr(zheight - zTopCut + halfCarTol) )
-      {
-        return distMin = std::fabs(lambda);
-      }
+    {
+      return distMin = std::fabs(lambda);
+    }
   }
-  
+
   if (p.z() > zTopCut - halfCarTol
    && p.z() < zTopCut + halfCarTol )
   {
-    if (v.z() > 0.) 
-      { return kInfinity; }
+    if (v.z() > 0.) { return kInfinity; }
 
     return distMin = 0.;
   }
-  
+
   if (p.z() < -zTopCut + halfCarTol
    && p.z() > -zTopCut - halfCarTol)
   {
-    if (v.z() < 0.)
-      { return distMin = kInfinity; }
-    
+    if (v.z() < 0.) { return distMin = kInfinity; }
+
     return distMin = 0.;
   }
-  
+
 #endif
 
-  // if we are here then it either intersects or grazes the curved surface 
+  // if we are here then it either intersects or grazes the curved surface
   // or it does not intersect at all
   //
   G4double A = sqr(v.x()/xSemiAxis) + sqr(v.y()/ySemiAxis) - sqr(v.z());
-  G4double B = 2*(v.x()*p.x()/sqr(xSemiAxis) + 
+  G4double B = 2*(v.x()*p.x()/sqr(xSemiAxis) +
                   v.y()*p.y()/sqr(ySemiAxis) + v.z()*(zheight-p.z()));
-  G4double C = sqr(p.x()/xSemiAxis) + sqr(p.y()/ySemiAxis) - 
+  G4double C = sqr(p.x()/xSemiAxis) + sqr(p.y()/ySemiAxis) -
                sqr(zheight - p.z());
- 
+
   G4double discr = B*B - 4.*A*C;
-   
+
   // if the discriminant is negative it never hits the curved object
   //
-  if ( discr < -halfCarTol )
-    { return distMin; }
-  
+  if ( discr < -halfCarTol ) { return distMin; }
+
   // case below is when it hits or grazes the surface
   //
   if ( (discr >= -halfCarTol ) && (discr < halfCarTol ) )
   {
-    return distMin = std::fabs(-B/(2.*A)); 
+    return distMin = std::fabs(-B/(2.*A));
   }
-  
+
   G4double plus  = (-B+std::sqrt(discr))/(2.*A);
   G4double minus = (-B-std::sqrt(discr))/(2.*A);
- 
+
   // Special case::Point on Surface, Check norm.dot(v)
 
   if ( ( std::fabs(plus) < halfCarTol )||( std::fabs(minus) < halfCarTol ) )
@@ -542,16 +572,13 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
     {
       return kInfinity;
     }
-    else
-    {
-      return 0;
-    }
+    return 0;
   }
 
-  // G4double lambda = std::fabs(plus) < std::fabs(minus) ? plus : minus;  
+  // G4double lambda = std::fabs(plus) < std::fabs(minus) ? plus : minus;
   G4double lambda = 0;
 
-  if ( minus > halfCarTol && minus < distMin ) 
+  if ( minus > halfCarTol && minus < distMin )
   {
     lambda = minus ;
     // check normal vector   n * v < 0
@@ -583,7 +610,7 @@ G4double G4EllipticalCone::DistanceToIn( const G4ThreeVector& p,
       }
     }
   }
-  if (distMin < halfCarTol) distMin=0.;
+  if (distMin < halfCarTol) { distMin=0.; }
   return distMin ;
 }
 
@@ -614,7 +641,7 @@ G4double G4EllipticalCone::DistanceToOut(const G4ThreeVector& p,
 {
   G4double distMin, lambda;
   enum surface_e {kPlaneSurf, kCurvedSurf, kNoSurf} surface;
-  
+
   distMin = kInfinity;
   surface = kNoSurf;
 
@@ -622,14 +649,14 @@ G4double G4EllipticalCone::DistanceToOut(const G4ThreeVector& p,
   {
     lambda = (-p.z() - zTopCut)/v.z();
 
-    if ( (sqr((p.x() + lambda*v.x())/xSemiAxis) + 
-          sqr((p.y() + lambda*v.y())/ySemiAxis)) < 
+    if ( (sqr((p.x() + lambda*v.x())/xSemiAxis) +
+          sqr((p.y() + lambda*v.y())/ySemiAxis)) <
           sqr(zheight + zTopCut + halfCarTol) )
     {
       distMin = std::fabs(lambda);
 
       if (!calcNorm) { return distMin; }
-    } 
+    }
     distMin = std::fabs(lambda);
     surface = kPlaneSurf;
   }
@@ -648,20 +675,20 @@ G4double G4EllipticalCone::DistanceToOut(const G4ThreeVector& p,
     distMin = std::fabs(lambda);
     surface = kPlaneSurf;
   }
-  
-  // if we are here then it either intersects or grazes the 
+
+  // if we are here then it either intersects or grazes the
   // curved surface...
   //
   G4double A = sqr(v.x()/xSemiAxis) + sqr(v.y()/ySemiAxis) - sqr(v.z());
-  G4double B = 2.*(v.x()*p.x()/sqr(xSemiAxis) +  
+  G4double B = 2.*(v.x()*p.x()/sqr(xSemiAxis) +
                    v.y()*p.y()/sqr(ySemiAxis) + v.z()*(zheight-p.z()));
   G4double C = sqr(p.x()/xSemiAxis) + sqr(p.y()/ySemiAxis)
              - sqr(zheight - p.z());
- 
+
   G4double discr = B*B - 4.*A*C;
-  
+
   if ( discr >= - halfCarTol && discr < halfCarTol )
-  { 
+  {
     if(!calcNorm) { return distMin = std::fabs(-B/(2.*A)); }
   }
 
@@ -679,7 +706,7 @@ G4double G4EllipticalCone::DistanceToOut(const G4ThreeVector& p,
     else
     {
       // at least one solution is close to zero or negative
-      // so, take small positive solution or zero 
+      // so, take small positive solution or zero
       //
       lambda   = plus > -halfCarTol ? plus : 0;
     }
@@ -701,7 +728,7 @@ G4double G4EllipticalCone::DistanceToOut(const G4ThreeVector& p,
           distMin  = 0.0;
           surface  = kCurvedSurf;
         }
-      } 
+      }
     }
   }
 
@@ -732,7 +759,7 @@ G4double G4EllipticalCone::DistanceToOut(const G4ThreeVector& p,
                                   -( pexit.z() - zheight ) );
           truenorm /= truenorm.mag();
           *n= truenorm;
-        } 
+        }
         break;
 
         default:            // Should never reach this case ...
@@ -840,77 +867,37 @@ std::ostream& G4EllipticalCone::StreamInfo( std::ostream& os ) const
 
 G4ThreeVector G4EllipticalCone::GetPointOnSurface() const
 {
-  G4double x0 = xSemiAxis*zheight; // x semi axis at z=0
-  G4double y0 = ySemiAxis*zheight; // y semi axis at z=0
-  G4double s0 = G4GeomTools::EllipticConeLateralArea(x0,y0,zheight);
-  G4double kmin = (zTopCut >= zheight ) ? 0. : (zheight - zTopCut)/zheight;
-  G4double kmax = (zTopCut >= zheight ) ? 2. : (zheight + zTopCut)/zheight;
-
-  // Set areas (base at -Z, side surface, base at +Z)
-  //
-  G4double szmin =  pi*x0*y0*kmax*kmax;
-  G4double szmax =  pi*x0*y0*kmin*kmin;
-  G4double sside =  s0*(kmax*kmax - kmin*kmin);
-  G4double ssurf[3] = { szmin, sside, szmax };
-  for (auto i=1; i<3; ++i) { ssurf[i] += ssurf[i-1]; }
-
-  // Select surface
-  //
-  G4double select = ssurf[2]*G4UniformRand();
-  G4int k = 2;
-  if (select <= ssurf[1]) k = 1;
-  if (select <= ssurf[0]) k = 0;
-
-  // Pick random point on selected surface
-  //
-  G4ThreeVector p;
-  switch(k)
+  G4double phi = twopi*G4QuickRand();
+  G4double cosphi = std::cos(phi);
+  G4double sinphi = std::sin(phi);
+  G4double select = fSurfaceArea*G4QuickRand();
+  if (select < fMinZBaseArea + fMaxZBaseArea)
   {
-    case 0: // base at -Z, uniform distribution, rejection sampling
-    {
-      G4double zh = zheight + zTopCut;
-      G4TwoVector rho = G4RandomPointInEllipse(zh*xSemiAxis,zh*ySemiAxis);
-      p.set(rho.x(),rho.y(),-zTopCut);
-      break;
-    }
-    case 1: // side surface, uniform distribution, rejection sampling
-    {
-      G4double zh = G4RandomRadiusInRing(zheight-zTopCut, zheight+zTopCut);
-      G4double a = x0;
-      G4double b = y0;
-
-      G4double hh = zheight*zheight;
-      G4double aa = a*a;
-      G4double bb = b*b;
-      G4double R  = std::max(a,b);
-      G4double mu_max = R*std::sqrt(hh + R*R);
-
-      G4double x,y;
-      for (auto i=0; i<1000; ++i)
-      {
-	G4double phi = CLHEP::twopi*G4UniformRand();
-        x = std::cos(phi);
-        y = std::sin(phi);
-        G4double xx = x*x;
-        G4double yy = y*y;
-        G4double E = hh + aa*xx + bb*yy;
-        G4double F = (aa-bb)*x*y;
-        G4double G = aa*yy + bb*xx;
-        G4double mu = std::sqrt(E*G - F*F);
-        if (mu_max*G4UniformRand() <= mu) break;
-      }
-      p.set(zh*xSemiAxis*x,zh*ySemiAxis*y,zheight-zh);
-      break;
-    }
-    case 2: // base at +Z, uniform distribution, rejection sampling
-    {
-      G4double zh = zheight - zTopCut;
-      G4TwoVector rho = G4RandomPointInEllipse(zh*xSemiAxis,zh*ySemiAxis);
-      p.set(rho.x(),rho.y(),zTopCut);
-      break;
-    }
+    // elliptical bases
+    G4double z = (select < fMinZBaseArea) ? -zTopCut : zTopCut;
+    G4double a = xSemiAxis*(zheight - z);
+    G4double b = ySemiAxis*(zheight - z);
+    G4double rho = std::sqrt(G4QuickRand());
+    return { a*rho*cosphi, b*rho*sinphi, z };
   }
-  return p;
+  
+  // lateral surface (rejection sampling)
+  G4double ucut = zTopCut/zheight;
+  G4double a = xSemiAxis;
+  G4double b = ySemiAxis;
+  G4double h = zheight;
+  G4double aa = a*a;
+  G4double bb = b*b;
+  G4double aabb = aa*bb + bb*cosphi*cosphi + aa*sinphi*sinphi;
+  G4double ss_max = 4.*ucut*ucut*(std::max(aa, bb) + aa*bb);
+  G4double u;
+  for (auto i = 0; i < 10000; ++i)
+  {
+    u = ucut*(2.*G4QuickRand() - 1.);
+    G4double ss = (1. - u)*(1. - u)*aabb;
+    if (ss_max*sqr(G4QuickRand()) <= ss) { break; }
+  }
+  return { a*h*(1. - u)*cosphi, b*h*(1 - u)*sinphi, h*u };
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -919,14 +906,16 @@ G4ThreeVector G4EllipticalCone::GetPointOnSurface() const
 
 G4double G4EllipticalCone::GetCubicVolume()
 {
-  if (fCubicVolume == 0.0)
+  if (fCubicVolume == 0)
   {
+    G4AutoLock l(&elconeMutex);
     G4double x0 = xSemiAxis*zheight; // x semi axis at z=0
     G4double y0 = ySemiAxis*zheight; // y semi axis at z=0
     G4double v0 = CLHEP::pi*x0*y0*zheight/3.;
     G4double kmin = (zTopCut >= zheight ) ? 0. : (zheight - zTopCut)/zheight;
     G4double kmax = (zTopCut >= zheight ) ? 2. : (zheight + zTopCut)/zheight;
     fCubicVolume = (kmax - kmin)*(kmax*kmax + kmax*kmin + kmin*kmin)*v0;
+    l.unlock();
   }
   return fCubicVolume;
 }
@@ -937,15 +926,18 @@ G4double G4EllipticalCone::GetCubicVolume()
 
 G4double G4EllipticalCone::GetSurfaceArea()
 {
-  if (fSurfaceArea == 0.0)
+  if (fSurfaceArea == 0)
   {
+    G4AutoLock l(&elconeMutex);
     G4double x0 = xSemiAxis*zheight; // x semi axis at z=0
     G4double y0 = ySemiAxis*zheight; // y semi axis at z=0
     G4double s0 = G4GeomTools::EllipticConeLateralArea(x0,y0,zheight);
     G4double kmin = (zTopCut >= zheight ) ? 0. : (zheight - zTopCut)/zheight;
     G4double kmax = (zTopCut >= zheight ) ? 2. : (zheight + zTopCut)/zheight;
-    fSurfaceArea = (kmax - kmin)*(kmax + kmin)*s0
-                 + CLHEP::pi*x0*y0*(kmin*kmin + kmax*kmax);
+    fMinZBaseArea =  pi*x0*y0*kmax*kmax;
+    fMaxZBaseArea =  pi*x0*y0*kmin*kmin;
+    fSurfaceArea = (kmax - kmin)*(kmax + kmin)*s0 + fMinZBaseArea + fMaxZBaseArea;
+    l.unlock();
   }
   return fSurfaceArea;
 }
