@@ -25,8 +25,8 @@
 //
 // G4EllipticalTube implementation
 //
-// Author: David C. Williams (davidw@scipp.ucsc.edu)
-// Revision: Evgueni Tcherniaev (evgueni.tcherniaev@cern.ch), 23.12.2019
+// Author: David C. Williams (UCSC), 29.03.2000 - First implementation
+//         Evgueni Tcherniaev (CERN), 23.12.2019 - Revised
 // --------------------------------------------------------------------
 
 #include "G4EllipticalTube.hh"
@@ -34,13 +34,10 @@
 #if !(defined(G4GEOM_USE_UELLIPTICALTUBE) && defined(G4GEOM_USE_SYS_USOLIDS))
 
 #include "G4GeomTools.hh"
-#include "G4RandomTools.hh"
-#include "G4ClippablePolygon.hh"
 #include "G4AffineTransform.hh"
 #include "G4VoxelLimits.hh"
 #include "G4BoundingEnvelope.hh"
-
-#include "Randomize.hh"
+#include "G4QuickRand.hh"
 
 #include "G4VGraphicsScene.hh"
 #include "G4VisExtent.hh"
@@ -50,6 +47,7 @@
 namespace
 {
   G4Mutex polyhedronMutex = G4MUTEX_INITIALIZER;
+  G4Mutex eltubeMutex = G4MUTEX_INITIALIZER;
 }
 
 using namespace CLHEP;
@@ -65,6 +63,7 @@ G4EllipticalTube::G4EllipticalTube( const G4String &name,
   : G4VSolid(name), fDx(Dx), fDy(Dy), fDz(Dz)
 {
   CheckParameters();
+  fSurfaceArea = GetCachedSurfaceArea();
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -106,7 +105,7 @@ G4EllipticalTube::G4EllipticalTube(const G4EllipticalTube& rhs)
 //
 // Assignment operator
 
-G4EllipticalTube& G4EllipticalTube::operator = (const G4EllipticalTube& rhs) 
+G4EllipticalTube& G4EllipticalTube::operator = (const G4EllipticalTube& rhs)
 {
    // Check assignment to self
    //
@@ -266,7 +265,7 @@ EInside G4EllipticalTube::Inside( const G4ThreeVector& p ) const
   G4double distZ = std::abs(p.z()) - fDz;
   G4double dist = std::max(distR, distZ);
 
-  if (dist > halfTolerance) return kOutside;
+  if (dist > halfTolerance) { return kOutside; }
   return (dist > -halfTolerance) ? kSurface : kInside;
 }
 
@@ -298,28 +297,32 @@ G4ThreeVector G4EllipticalTube::SurfaceNormal( const G4ThreeVector& p ) const
   }
 
   // return normal
-  if (nsurf == 1) return norm;
-  else if (nsurf > 1) return norm.unit(); // edge
-  else
+  if (nsurf == 1)
   {
-    // Point is not on the surface
-    //
-#ifdef G4SPECDEBUG
-    std::ostringstream message;
-    G4long oldprc = message.precision(16);
-    message << "Point p is not on surface (!?) of solid: "
-            << GetName() << G4endl;
-    message << "Position:\n";
-    message << "   p.x() = " << p.x()/mm << " mm\n";
-    message << "   p.y() = " << p.y()/mm << " mm\n";
-    message << "   p.z() = " << p.z()/mm << " mm";
-    G4cout.precision(oldprc);
-    G4Exception("G4EllipticalTube::SurfaceNormal(p)", "GeomSolids1002",
-                JustWarning, message );
-    DumpInfo();
-#endif
-    return ApproxSurfaceNormal(p);
+    return norm;
   }
+  if (nsurf > 1)
+  {
+    return norm.unit(); // edge
+  }
+
+  // Point is not on the surface
+  //
+#ifdef G4SPECDEBUG
+  std::ostringstream message;
+  G4long oldprc = message.precision(16);
+  message << "Point p is not on surface (!?) of solid: "
+          << GetName() << G4endl;
+  message << "Position:\n";
+  message << "   p.x() = " << p.x()/mm << " mm\n";
+  message << "   p.y() = " << p.y()/mm << " mm\n";
+  message << "   p.z() = " << p.z()/mm << " mm";
+  G4cout.precision(oldprc);
+  G4Exception("G4EllipticalTube::SurfaceNormal(p)", "GeomSolids1002",
+              JustWarning, message );
+  DumpInfo();
+#endif
+  return ApproxSurfaceNormal(p);
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -336,9 +339,10 @@ G4EllipticalTube::ApproxSurfaceNormal( const G4ThreeVector& p ) const
   G4double distR = fQ1 * (x * x + y * y) - fQ2;
   G4double distZ = std::abs(p.z()) - fDz;
   if (distR > distZ && (x * x + y * y) > 0)
+  {
     return G4ThreeVector(p.x() * fDDy, p.y() * fDDx, 0.).unit();
-  else
-    return {0, 0, (p.z() < 0 ? -1. : 1.)};
+  }
+  return {0, 0, (p.z() < 0 ? -1. : 1.)};
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -358,9 +362,9 @@ G4double G4EllipticalTube::DistanceToIn( const G4ThreeVector& p,
   G4double safey = std::abs(pcur.y()) - fDy;
   G4double safez = std::abs(pcur.z()) - fDz;
 
-  if (safez >= -halfTolerance && pcur.z() * v.z() >= 0.) return kInfinity;
-  if (safey >= -halfTolerance && pcur.y() * v.y() >= 0.) return kInfinity;
-  if (safex >= -halfTolerance && pcur.x() * v.x() >= 0.) return kInfinity;
+  if (safez >= -halfTolerance && pcur.z() * v.z() >= 0.) { return kInfinity; }
+  if (safey >= -halfTolerance && pcur.y() * v.y() >= 0.) { return kInfinity; }
+  if (safex >= -halfTolerance && pcur.x() * v.x() >= 0.) { return kInfinity; }
 
   // Relocate point, if required
   //
@@ -394,7 +398,7 @@ G4double G4EllipticalTube::DistanceToIn( const G4ThreeVector& p,
   //
   G4double distR  = fQ1 * rr - fQ2;
   G4bool parallelToZ = (A < DBL_EPSILON || std::abs(vz) >= 1.);
-  if (distR >= -halfTolerance && (B >= 0. || parallelToZ)) return kInfinity;
+  if (distR >= -halfTolerance && (B >= 0. || parallelToZ)) { return kInfinity; }
 
   // Find intersection with Z planes
   //
@@ -407,8 +411,11 @@ G4double G4EllipticalTube::DistanceToIn( const G4ThreeVector& p,
   //   1) trajectory parallel to Z axis (A = 0, B = 0, C - any, D = 0)
   //   2) touch (D = 0) or no intersection (D < 0) with lateral surface
   //
-  if (parallelToZ) return (tzmin<halfTolerance) ? offset : tzmin + offset; // 1)
-  if (D <= A * A * fScratch) return kInfinity; // 2)
+  if (parallelToZ)
+  {
+    return (tzmin<halfTolerance) ? offset : tzmin + offset;  // 1)
+  }
+  if (D <= A * A * fScratch) { return kInfinity; } // 2)
 
   // Find roots of quadratic equation
   G4double tmp = -B - std::copysign(std::sqrt(D), B);
@@ -421,7 +428,8 @@ G4double G4EllipticalTube::DistanceToIn( const G4ThreeVector& p,
   G4double tin  = std::max(tzmin, trmin);
   G4double tout = std::min(tzmax, trmax);
 
-  if (tout <= tin + halfTolerance) return kInfinity; // touch or no hit
+  if (tout <= tin + halfTolerance) { return kInfinity; } // touch or no hit
+
   return (tin<halfTolerance) ? offset : tin + offset;
 }
 
@@ -569,9 +577,13 @@ G4double G4EllipticalTube::DistanceToOut( const G4ThreeVector& p,
     *validNorm = true;
     G4ThreeVector pnew = p + tmax * v;
     if (tmax == tzmax)
+    {
       n->set(0, 0, (pnew.z() < 0) ? -1. : 1.);
+    }
     else
+    {
       *n = G4ThreeVector(pnew.x() * fDDy, pnew.y() * fDDx, 0.).unit();
+    }
   }
   return tmax;
 }
@@ -631,19 +643,6 @@ G4VSolid* G4EllipticalTube::Clone() const
   return new G4EllipticalTube(*this);
 }
 
-//////////////////////////////////////////////////////////////////////////
-//
-// Return volume
-
-G4double G4EllipticalTube::GetCubicVolume()
-{
-  if (fCubicVolume == 0.)
-  {
-    fCubicVolume = twopi * fDx * fDy * fDz;
-  }
-  return fCubicVolume;
-}
-
 //////////////////////////////////////////////////////////////////////////
 //
 // Return cached surface area
@@ -664,15 +663,32 @@ G4double G4EllipticalTube::GetCachedSurfaceArea() const
   return cached_area;
 }
 
+//////////////////////////////////////////////////////////////////////////
+//
+// Return volume
+
+G4double G4EllipticalTube::GetCubicVolume()
+{
+  if (fCubicVolume == 0)
+  {
+    G4AutoLock l(&eltubeMutex);
+    fCubicVolume = twopi * fDx * fDy * fDz;
+    l.unlock();
+  }
+  return fCubicVolume;
+}
+
 //////////////////////////////////////////////////////////////////////////
 //
 // Return surface area
 
 G4double G4EllipticalTube::GetSurfaceArea()
 {
-  if(fSurfaceArea == 0.)
+  if(fSurfaceArea == 0)
   {
+    G4AutoLock l(&eltubeMutex);
     fSurfaceArea = GetCachedSurfaceArea();
+    l.unlock();
   }
   return fSurfaceArea;
 }
@@ -701,44 +717,37 @@ std::ostream& G4EllipticalTube::StreamInfo(std::ostream& os) const
 
 //////////////////////////////////////////////////////////////////////////
 //
-// Pick up a random point on the surface 
+// Pick up a random point on the surface
 
 G4ThreeVector G4EllipticalTube::GetPointOnSurface() const
 {
-  // Select surface (0 - base at -Z, 1 - base at +Z, 2 - lateral surface)
+  // Pick random point on selected surface
   //
-  G4double sbase = pi * fDx * fDy;
-  G4double ssurf = GetCachedSurfaceArea();
-  G4double select = ssurf * G4UniformRand();
-
-  G4int k = 0;
-  if (select > sbase) k = 1;
-  if (select > 2. * sbase) k = 2;
-
-  // Pick random point on selected surface (rejection sampling)
-  //
-  G4ThreeVector p;
-  switch (k) {
-    case 0: // base at -Z
-    {
-      G4TwoVector rho = G4RandomPointInEllipse(fDx, fDy);
-      p.set(rho.x(), rho.y(), -fDz);
-      break;
-    }
-    case 1: // base at +Z
-    {
-      G4TwoVector rho = G4RandomPointInEllipse(fDx, fDy);
-      p.set(rho.x(), rho.y(), fDz);
-      break;
-    }
-    case 2: // lateral surface
-    {
-      G4TwoVector rho = G4RandomPointOnEllipse(fDx, fDy);
-      p.set(rho.x(), rho.y(), (2. * G4UniformRand() - 1.) * fDz);
-      break;
-    }
+  G4double sbase = pi * fDx * fDy; // base area
+  G4double select = fSurfaceArea * G4QuickRand();
+  G4double x, y, z;
+  if (select < 2. * sbase)
+  { // base (ellipse)
+    G4double phi = CLHEP::twopi * G4QuickRand();
+    G4double rho = std::sqrt(G4QuickRand());
+    x = rho * std::cos(phi);
+    y = rho * std::sin(phi);
+    z = (select < sbase) ? fDz : -fDz;
   }
-  return p;
+  else
+  { // lateral surface (rejection sampling)
+    G4double s_max = std::max(fDx, fDy);
+    for (auto i = 0; i < 10000; ++i)
+    {
+      G4double phi = CLHEP::twopi * G4QuickRand();
+      x = std::cos(phi);
+      y = std::sin(phi);
+      G4double ss = sqr(fDy * x) + sqr(fDx * y);
+      if (sqr(s_max*G4QuickRand()) <= ss) { break; }
+    }
+    z = (2. * G4QuickRand() - 1.) * fDz;
+  }
+  return { fDx * x, fDy * y, z };
 }
 
 
