@@ -50,6 +50,7 @@
 namespace
 {
   G4Mutex surface_elementsMutex = G4MUTEX_INITIALIZER;
+  G4Mutex genpolyMutex = G4MUTEX_INITIALIZER;
 }
 
 using namespace CLHEP;
@@ -78,8 +79,7 @@ G4GenericPolycone::G4GenericPolycone( const G4String& name,
 
 // Create
 //
-// Generic create routine, called by each constructor after
-// conversion of arguments
+// Generic create routine, called by constructor after conversion of arguments
 //
 void G4GenericPolycone::Create( G4double phiStart,
                                 G4double phiTotal,
@@ -151,11 +151,15 @@ void G4GenericPolycone::Create( G4double phiStart,
     //
     startPhi = phiStart;
     while( startPhi < 0 )    // Loop checking, 13.08.2015, G.Cosmo
+    {
       startPhi += twopi;
+    }
 
     endPhi = phiStart+phiTotal;
     while( endPhi < startPhi )    // Loop checking, 13.08.2015, G.Cosmo
+    {
       endPhi += twopi;
+    }
   }
 
   //
@@ -194,11 +198,11 @@ void G4GenericPolycone::Create( G4double phiStart,
   do    // Loop checking, 13.08.2015, G.Cosmo
   {
     next = corner+1;
-    if (next >= corners+numCorner) next = corners;
+    if (next >= corners+numCorner) { next = corners; }
     nextNext = next+1;
-    if (nextNext >= corners+numCorner) nextNext = corners;
+    if (nextNext >= corners+numCorner) { nextNext = corners; }
 
-    if (corner->r < 1/kInfinity && next->r < 1/kInfinity) continue;
+    if (corner->r < 1/kInfinity && next->r < 1/kInfinity) { continue; }
 
     //
     // We must decide here if we can dare declare one of our faces
@@ -281,7 +285,7 @@ G4GenericPolycone::G4GenericPolycone( const G4GenericPolycone& source )
 G4GenericPolycone&
 G4GenericPolycone::operator=( const G4GenericPolycone& source )
 {
-  if (this == &source) return *this;
+  if (this == &source) { return *this; }
 
   G4VCSGfaceted::operator=( source );
 
@@ -359,7 +363,8 @@ EInside G4GenericPolycone::Inside( const G4ThreeVector& p ) const
   //
   // Quick test
   //
-  if (enclosingCylinder->MustBeOutside(p)) return kOutside;
+  if (enclosingCylinder->MustBeOutside(p)) { return kOutside;
+}
 
   //
   // Long answer
@@ -378,8 +383,7 @@ G4double G4GenericPolycone::DistanceToIn( const G4ThreeVector& p,
   //
   // Quick test
   //
-  if (enclosingCylinder->ShouldMiss(p,v))
-    return kInfinity;
+  if (enclosingCylinder->ShouldMiss(p,v)) { return kInfinity; }
 
   //
   // Long answer
@@ -408,10 +412,10 @@ G4GenericPolycone::BoundingLimits(G4ThreeVector& pMin,
   for (G4int i=0; i<GetNumRZCorner(); ++i)
   {
     G4PolyconeSideRZ corner = GetCorner(i);
-    if (corner.r < rmin) rmin = corner.r;
-    if (corner.r > rmax) rmax = corner.r;
-    if (corner.z < zmin) zmin = corner.z;
-    if (corner.z > zmax) zmax = corner.z;
+    if (corner.r < rmin) { rmin = corner.r; }
+    if (corner.r > rmax) { rmax = corner.r; }
+    if (corner.z < zmin) { zmin = corner.z; }
+    if (corner.z > zmax) { zmax = corner.z; }
   }
 
   if (IsOpen())
@@ -486,7 +490,7 @@ G4GenericPolycone::CalculateExtent(const EAxis pAxis,
     contourRZ.emplace_back(corner.r,corner.z);
   }
   G4double area = G4GeomTools::PolygonArea(contourRZ);
-  if (area < 0.) std::reverse(contourRZ.begin(),contourRZ.end());
+  if (area < 0.) { std::reverse(contourRZ.begin(),contourRZ.end()); }
 
   // triangulate RZ countour
   if (!G4GeomTools::TriangulatePolygon(contourRZ,triangles))
@@ -524,8 +528,14 @@ G4GenericPolycone::CalculateExtent(const EAxis pAxis,
   std::vector<const G4ThreeVectorList *> polygons;
   polygons.resize(ksteps+2);
   G4ThreeVectorList pols[NSTEPS+2];
-  for (G4int k=0; k<ksteps+2; ++k) pols[k].resize(6);
-  for (G4int k=0; k<ksteps+2; ++k) polygons[k] = &pols[k];
+  for (G4int k=0; k<ksteps+2; ++k)
+  {
+    pols[k].resize(6);
+  }
+  for (G4int k=0; k<ksteps+2; ++k)
+  {
+    polygons[k] = &pols[k];
+  }
   G4double r0[6],z0[6]; // contour with original edges of triangle
   G4double r1[6];       // shifted radii of external edges of triangle
 
@@ -546,7 +556,7 @@ G4GenericPolycone::CalculateExtent(const EAxis pAxis,
       // set shifted radii
       r1[k2+0] = r0[k2+0];
       r1[k2+1] = r0[k2+1];
-      if (z0[k2+1] - z0[k2+0] <= 0) continue;
+      if (z0[k2+1] - z0[k2+0] <= 0) { continue; }
       r1[k2+0] /= cosHalf;
       r1[k2+1] /= cosHalf;
     }
@@ -576,10 +586,13 @@ G4GenericPolycone::CalculateExtent(const EAxis pAxis,
     // set sub-envelope and adjust extent
     G4double emin,emax;
     G4BoundingEnvelope benv(polygons);
-    if (!benv.CalculateExtent(pAxis,pVoxelLimit,pTransform,emin,emax)) continue;
-    if (emin < pMin) pMin = emin;
-    if (emax > pMax) pMax = emax;
-    if (eminlim > pMin && emaxlim < pMax) return true; // max possible extent
+    if (!benv.CalculateExtent(pAxis,pVoxelLimit,pTransform,emin,emax))
+    {
+      continue;
+    }
+    if (emin < pMin) { pMin = emin; }
+    if (emax > pMax) { pMax = emax; }
+    if (eminlim > pMin && emaxlim < pMax) { return true; } // max possible extent
   }
   return (pMin < pMax);
 }
@@ -618,11 +631,11 @@ std::ostream& G4GenericPolycone::StreamInfo( std::ostream& os ) const
 
   os << "    number of RZ points: " << numCorner << "\n"
      << "              RZ values (corners): \n";
-     for (i=0; i<numCorner; i++)
-     {
-       os << "                         "
-          << corners[i].r << ", " << corners[i].z << "\n";
-     }
+  for (i=0; i<numCorner; ++i)
+  {
+    os << "                         "
+       << corners[i].r << ", " << corners[i].z << "\n";
+  }
   os << "-----------------------------------------------------------\n";
   os.precision(oldprc);
 
@@ -635,8 +648,9 @@ std::ostream& G4GenericPolycone::StreamInfo( std::ostream& os ) const
 
 G4double G4GenericPolycone::GetCubicVolume()
 {
-  if (fCubicVolume == 0.)
+  if (fCubicVolume == 0)
   {
+    G4AutoLock l(&genpolyMutex);
     G4double total = 0.;
     G4int nrz = GetNumRZCorner();
     G4PolyconeSideRZ a = GetCorner(nrz - 1);
@@ -647,6 +661,7 @@ G4double G4GenericPolycone::GetCubicVolume()
       a = b;
     }
     fCubicVolume = std::abs(total)*(GetEndPhi() - GetStartPhi())/6.;
+    l.unlock();
   }
   return fCubicVolume;
 }
@@ -657,8 +672,9 @@ G4double G4GenericPolycone::GetCubicVolume()
 
 G4double G4GenericPolycone::GetSurfaceArea()
 {
-  if (fSurfaceArea == 0.)
+  if (fSurfaceArea == 0)
   {
+    G4AutoLock l(&genpolyMutex);
     // phi cut area
     G4int nrz = GetNumRZCorner();
     G4double scut = 0.;
@@ -685,6 +701,7 @@ G4double G4GenericPolycone::GetSurfaceArea()
     }
     slat *= (GetEndPhi() - GetStartPhi())/2.;
     fSurfaceArea = scut + slat;
+    l.unlock();
   }
   return fSurfaceArea;
 }
@@ -712,7 +729,7 @@ void G4GenericPolycone::SetSurfaceElements() const
     selem.i1 = ib;
     selem.i2 = -1;
     ia = ib;
-    if (a.r == 0. && b.r == 0.) continue;
+    if (a.r == 0. && b.r == 0.) { continue; }
     G4double h = std::sqrt((b.r - a.r)*(b.r - a.r) + (b.z - a.z)*(b.z - a.z));
     sarea += 0.5*dphi*(b.r + a.r)*h;
     selem.area = sarea;
@@ -826,7 +843,9 @@ G4Polyhedron* G4GenericPolycone::CreatePolyhedron() const
 {
   std::vector<G4TwoVector> rz(numCorner);
   for (G4int i = 0; i < numCorner; ++i)
+  {
     rz[i].set(corners[i].r, corners[i].z);
+  }
   return new G4PolyhedronPcon(startPhi, endPhi - startPhi, rz);
 }
 
