@@ -38,10 +38,7 @@
 #include "G4VoxelLimits.hh"
 #include "G4AffineTransform.hh"
 #include "G4BoundingEnvelope.hh"
-
-#include "meshdefs.hh"
-
-#include "Randomize.hh"
+#include "G4QuickRand.hh"
 
 #include "G4VGraphicsScene.hh"
 #include "G4VisExtent.hh"
@@ -51,6 +48,7 @@
 namespace
 {
   G4Mutex polyhedronMutex = G4MUTEX_INITIALIZER;
+  G4Mutex paraboloidMutex = G4MUTEX_INITIALIZER;
 }
 
 using namespace CLHEP;
@@ -70,7 +68,7 @@ G4Paraboloid::G4Paraboloid(const G4String& pName,
     std::ostringstream message;
     message << "Invalid dimensions. Negative Input Values or R1>=R2 - "
             << GetName();
-    G4Exception("G4Paraboloid::G4Paraboloid()", "GeomSolids0002", 
+    G4Exception("G4Paraboloid::G4Paraboloid()", "GeomSolids0002",
                 FatalErrorInArgument, message,
                 "Z half-length must be larger than zero or R1>=R2.");
   }
@@ -86,6 +84,8 @@ G4Paraboloid::G4Paraboloid(const G4String& pName,
 
   k1 = (r2 * r2 - r1 * r1) / 2 / dz;
   k2 = (r2 * r2 + r1 * r1) / 2;
+
+  fSurfaceArea = CalculateSurfaceArea();
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -94,7 +94,7 @@ G4Paraboloid::G4Paraboloid(const G4String& pName,
 //                            for usage restricted to object persistency.
 //
 G4Paraboloid::G4Paraboloid( __void__& a )
-  : G4VSolid(a), dz(0.), r1(0.), r2(0.), k1(0.), k2(0.)
+  : G4VSolid(a)
 {
 }
 
@@ -122,7 +122,7 @@ G4Paraboloid::G4Paraboloid(const G4Paraboloid& rhs)
 //
 // Assignment operator
 //
-G4Paraboloid& G4Paraboloid::operator = (const G4Paraboloid& rhs) 
+G4Paraboloid& G4Paraboloid::operator = (const G4Paraboloid& rhs)
 {
    // Check assignment to self
    //
@@ -142,6 +142,127 @@ G4Paraboloid& G4Paraboloid::operator = (const G4Paraboloid& rhs)
    return *this;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+//
+// Set half height
+//
+void G4Paraboloid::SetZHalfLength(G4double pDz)
+{
+  if(pDz <= 0)
+  {
+    G4Exception("G4Paraboloid::SetZHalfLength()", "GeomSolids0002",
+                FatalException, "Invalid dimensions.");
+  }
+  else
+  {
+    dz = pDz;
+    k1 = (sqr(r2) - sqr(r1)) / (2 * dz);
+    k2 = (sqr(r2) + sqr(r1)) / 2;
+    fSurfaceArea = CalculateSurfaceArea();
+    fCubicVolume = 0.;
+    fRebuildPolyhedron = true;
+  }
+}
+
+///////////////////////////////////////////////////////////////////////////////
+//
+// Set radius at z = dz
+//
+void G4Paraboloid::SetRadiusPlusZ(G4double pR2)
+{
+  if(pR2 <= 0 || pR2 <= r1)
+  {
+    G4Exception("G4Paraboloid::SetRadiusPlusZ()", "GeomSolids0002",
+                FatalException, "Invalid dimensions.");
+  }
+  else
+  {
+    r2 = pR2;
+    k1 = (sqr(r2) - sqr(r1)) / (2 * dz);
+    k2 = (sqr(r2) + sqr(r1)) / 2;
+    fSurfaceArea = CalculateSurfaceArea();
+    fCubicVolume = 0.;
+    fRebuildPolyhedron = true;
+  }
+}
+
+///////////////////////////////////////////////////////////////////////////////
+//
+// Set radius at z = -dz
+//
+void G4Paraboloid::SetRadiusMinusZ(G4double pR1)
+{
+  if(pR1 < 0 || pR1 >= r2)
+  {
+    G4Exception("G4Paraboloid::SetRadiusMinusZ()", "GeomSolids0002",
+                FatalException, "Invalid dimensions.");
+  }
+  else
+  {
+    r1 = pR1;
+    k1 = (sqr(r2) - sqr(r1)) / (2 * dz);
+    k2 = (sqr(r2) + sqr(r1)) / 2;
+    fSurfaceArea = CalculateSurfaceArea();
+    fCubicVolume = 0.;
+    fRebuildPolyhedron = true;
+  }
+}
+
+///////////////////////////////////////////////////////////////////////////////
+//
+// Compute surface area of the solid
+//
+G4double G4Paraboloid::CalculateSurfaceArea() const
+{
+  // Surface area for a paraboloid of height hmax
+  G4double hmax = k2/k1 + dz;
+  G4double Amax = sqr(r2) + 4*sqr(hmax);
+  Amax *= std::sqrt(Amax); // set Amax = sqrt(Amax^3)
+  Amax = CLHEP::pi * r2 / 6 / sqr(hmax) * (Amax - r2 * r2 * r2);
+
+  // Surface area for a paraboloid of height hmin
+  G4double Amin = 0.0;
+  if(r1 != 0)
+  {
+    G4double hmin = k2/k1 - dz;
+    Amin = sqr(r1) + 4*sqr(hmin);
+    Amin *= std::sqrt(Amin); // set Amin = sqrt(Amin^3)
+    Amin = CLHEP::pi * r1 / 6 / sqr(hmin) * (Amin - r1 * r1 * r1);
+  }
+  // Total surface area
+  return Amax - Amin + (sqr(r1) + sqr(r2))*CLHEP::pi;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+//
+// Return surface area of the solid
+//
+G4double G4Paraboloid::GetSurfaceArea()
+{
+  if (fSurfaceArea == 0)
+  {
+    G4AutoLock l(&paraboloidMutex);
+    fSurfaceArea = CalculateSurfaceArea();
+    l.unlock();
+  }
+  return fSurfaceArea;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+//
+// Return volume of the solid
+//
+G4double G4Paraboloid::GetCubicVolume()
+{
+  if(fCubicVolume == 0)
+  {
+    G4AutoLock l(&paraboloidMutex);
+    fCubicVolume = CLHEP::pi * (sqr(r1) + sqr(r2)) * dz;
+    l.unlock();
+  }
+  return fCubicVolume;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Get bounding box
@@ -200,12 +321,12 @@ EInside G4Paraboloid::Inside(const G4ThreeVector& p) const
   G4double rho2 = p.perp2(),
            rhoSurfTimesTol2  = (k1 * p.z() + k2) * sqr(kCarTolerance),
            A = rho2 - ((k1 *p.z() + k2) + 0.25 * kCarTolerance * kCarTolerance);
- 
+
   if(A < 0 && sqr(A) > rhoSurfTimesTol2)
   {
     // Actually checking rho < radius of paraboloid at z = p.z().
     // We're either inside or in lower/upper cutoff area.
-   
+
     if(std::fabs(p.z()) > dz - 0.5 * kCarTolerance)
     {
       // We're in the upper/lower cutoff area, sides have a paraboloid shape
@@ -213,21 +334,16 @@ EInside G4Paraboloid::Inside(const G4ThreeVector& p) const
 
       return kSurface;
     }
-    else
-    {
-      return kInside;
-    }
+    return kInside;
   }
-  else if(A <= 0 || sqr(A) < rhoSurfTimesTol2)
+  if(A <= 0 || sqr(A) < rhoSurfTimesTol2)
   {
     // We're in the parabolic surface.
 
     return kSurface;
   }
-  else
-  {
-    return kOutside;
-  }
+  
+  return kOutside;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -335,7 +451,7 @@ G4double G4Paraboloid::DistanceToIn( const G4ThreeVector& p,
   G4double tol2 = kCarTolerance*kCarTolerance;
   G4double tolh = 0.5*kCarTolerance;
 
-  if((r2 != 0.0) && p.z() > - tolh + dz) 
+  if((r2 != 0.0) && p.z() > - tolh + dz)
   {
     // If the points is above check for intersection with upper edge.
 
@@ -345,10 +461,8 @@ G4double G4Paraboloid::DistanceToIn( const G4ThreeVector& p,
       if(sqr(p.x() + v.x()*intersection)
        + sqr(p.y() + v.y()*intersection) < sqr(r2 + 0.5 * kCarTolerance))
       {
-        if(p.z() < tolh + dz)
-          { return 0; }
-        else
-          { return intersection; }
+        if(p.z() < tolh + dz) { return 0; }
+        return intersection;
       }
     }
     else  // Direction away, no possibility of intersection
@@ -366,14 +480,8 @@ G4double G4Paraboloid::DistanceToIn( const G4ThreeVector& p,
       if(sqr(p.x() + v.x()*intersection)
        + sqr(p.y() + v.y()*intersection) < sqr(r1 + 0.5 * kCarTolerance))
       {
-        if(p.z() > -tolh - dz)
-        {
-          return 0;
-        }
-        else
-        {
-          return intersection;
-        }
+        if(p.z() > -tolh - dz) { return 0; }
+        return intersection;
       }
     }
     else  // Direction away, no possibility of intersection
@@ -392,68 +500,57 @@ G4double G4Paraboloid::DistanceToIn( const G4ThreeVector& p,
   {
     // Is there a problem with squaring rho twice?
 
-    if(vRho2<tol2) // Needs to be treated seperately.
+    if(vRho2<tol2) // Needs to be treated separately.
     {
       intersection = ((rho2 - k2)/k1 - p.z())/v.z();
       if(intersection < 0) { return kInfinity; }
-      else if(std::fabs(p.z() + v.z() * intersection) <= dz)
-      {
-        return intersection;
-      }
-      else
-      {
-        return kInfinity;
-      }
+      if(std::fabs(p.z() + v.z() * intersection) <= dz) { return intersection; }
+      return kInfinity;
     }
-    else if(A*A + B < 0) // No real intersections.
+
+    if(A*A + B < 0) // No real intersections.
     {
       return kInfinity;
     }
-    else
+    
+    intersection = (A - std::sqrt(B + sqr(A))) / vRho2;
+    if(intersection < 0)
     {
-      intersection = (A - std::sqrt(B + sqr(A))) / vRho2;
-      if(intersection < 0)
-      {
-        return kInfinity;
-      }
-      else if(std::fabs(p.z() + intersection * v.z()) < dz + tolh)
-      {
-        return intersection;
-      }
-      else
-      {
-        return kInfinity;
-      }
+      return kInfinity;
     }
+
+    if(std::fabs(p.z() + intersection * v.z()) < dz + tolh)
+    {
+      return intersection;
+    }
+
+    return kInfinity;
   }
-  else if(sqr(rho2 - paraRho2 - .25 * tol2) <= tol2 * paraRho2)
+  if(sqr(rho2 - paraRho2 - .25 * tol2) <= tol2 * paraRho2)
   {
     // If this is true we're somewhere in the border.
 
     G4ThreeVector normal(p.x(), p.y(), -k1/2);
-    if(normal.dot(v) <= 0)
-      { return 0; }
-    else
-      { return kInfinity; }
+    if(normal.dot(v) <= 0) { return 0; }
+    
+    return kInfinity;
+  }
+  
+  std::ostringstream message;
+  if(Inside(p) == kInside)
+  {
+    message << "Point p is inside! - " << GetName() << G4endl;
   }
   else
   {
-    std::ostringstream message;
-    if(Inside(p) == kInside)
-    {
-      message << "Point p is inside! - " << GetName() << G4endl;
-    }
-    else
-    {
-      message << "Likely a problem in this function, for solid: " << GetName()
-              << G4endl;
-    }
-    message << "          p = " << p * (1/mm) << " mm" << G4endl
-            << "          v = " << v * (1/mm) << " mm";
-    G4Exception("G4Paraboloid::DistanceToIn(p,v)", "GeomSolids1002",
-                JustWarning, message);
-    return 0;
+    message << "Likely a problem in this function, for solid: " << GetName()
+            << G4endl;
   }
+  message << "          p = " << p * (1/mm) << " mm" << G4endl
+          << "          v = " << v * (1/mm) << " mm";
+  G4Exception("G4Paraboloid::DistanceToIn(p,v)", "GeomSolids1002",
+              JustWarning, message);
+  return 0;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -595,11 +692,11 @@ G4double G4Paraboloid::DistanceToOut(const G4ThreeVector& p,
       }
       return intersection;
     }
-    else if( ((A <= 0) && (B >= sqr(A) * (sqr(vRho2) - 1))) || (A >= 0))
+    if( ((A <= 0) && (B >= sqr(A) * (sqr(vRho2) - 1))) || (A >= 0))
     {
       // intersection = (A + std::sqrt(B + sqr(A))) / vRho2;
       // The above calculation has a precision problem:
-      // known problem of solving quadratic equation with small A  
+      // known problem of solving quadratic equation with small A
 
       A = A/vRho2;
       B = (k1 * p.z() + k2 - rho2)/vRho2;
@@ -623,12 +720,12 @@ G4double G4Paraboloid::DistanceToOut(const G4ThreeVector& p,
 
     return kInfinity;
   }
-  else if ( (rho2 < paraRho2 + kCarTolerance
-         || sqr(rho2 - paraRho2 - 0.25 * tol2) < tol2 * paraRho2 )
-         && std::fabs(p.z()) < dz + tolh) 
+  if ( (rho2 < paraRho2 + kCarTolerance
+     || sqr(rho2 - paraRho2 - 0.25 * tol2) < tol2 * paraRho2 )
+     && std::fabs(p.z()) < dz + tolh)
   {
     // If this is true we're somewhere in the border.
-    
+
     G4ThreeVector normal = G4ThreeVector (p.x(), p.y(), -k1/2);
 
     if(std::fabs(p.z()) > dz - tolh)
@@ -671,7 +768,7 @@ G4double G4Paraboloid::DistanceToOut(const G4ThreeVector& p,
       }
     }
     //
-    // Problem in the Logic :: Following condition for point on upper surface 
+    // Problem in the Logic :: Following condition for point on upper surface
     //                         and Vz<0  will return 0 (Problem #1015), but
     //                         it has to return intersection with parabolic
     //                         surface or with lower plane surface (z = -dz)
@@ -707,7 +804,7 @@ G4double G4Paraboloid::DistanceToOut(const G4ThreeVector& p,
         }
         return intersection;
       }
-      else if(ip.perp2() < sqr(r2 + tolh))
+      if(ip.perp2() < sqr(r2 + tolh))
       {
         if(calcNorm)
         {
@@ -735,7 +832,7 @@ G4double G4Paraboloid::DistanceToOut(const G4ThreeVector& p,
         }
         return intersection;
       }
-      else if(ip.perp2() < sqr(r1 + tolh))
+      if(ip.perp2() < sqr(r1 + tolh))
       {
         if(calcNorm)
         {
@@ -787,22 +884,18 @@ G4double G4Paraboloid::DistanceToOut(const G4ThreeVector& p,
     }
     return intersection;
   }
-  else
-  {
+  
 #ifdef G4SPECSDEBUG
-    if(kOutside == Inside(p))
-    {
-      G4Exception("G4Paraboloid::DistanceToOut(p,v,...)", "GeomSolids1002",
-                  JustWarning, "Point p is outside!");
-    }
-    else
-      G4Exception("G4Paraboloid::DistanceToOut(p,v,...)", "GeomSolids1002",
-                  JustWarning, "There's an error in this functions code.");
-#endif
-    return kInfinity;
+  if(kOutside == Inside(p))
+  {
+    G4Exception("G4Paraboloid::DistanceToOut(p,v,...)", "GeomSolids1002",
+                JustWarning, "Point p is outside!");
   }
-  return 0;
-} 
+  G4Exception("G4Paraboloid::DistanceToOut(p,v,...)", "GeomSolids1002",
+              JustWarning, "There's an error in this functions code.");
+#endif
+  return kInfinity;
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 //
@@ -845,7 +938,7 @@ G4double G4Paraboloid::DistanceToOut(const G4ThreeVector& p) const
   return safe ;
 }
 
-//////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
 //
 // G4EntityType
 //
@@ -854,7 +947,7 @@ G4GeometryType G4Paraboloid::GetEntityType() const
   return {"G4Paraboloid"};
 }
 
-//////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
 //
 // Make a clone of the object
 //
@@ -863,7 +956,7 @@ G4VSolid* G4Paraboloid::Clone() const
   return new G4Paraboloid(*this);
 }
 
-//////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
 //
 // Stream object contents to an output stream
 //
@@ -884,38 +977,44 @@ std::ostream& G4Paraboloid::StreamInfo( std::ostream& os ) const
   return os;
 }
 
-////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
 //
 // GetPointOnSurface
 //
 G4ThreeVector G4Paraboloid::GetPointOnSurface() const
 {
-  G4double A = (fSurfaceArea == 0)? CalculateSurfaceArea(): fSurfaceArea;
-  G4double z = G4RandFlat::shoot(0.,1.);
-  G4double phi = G4RandFlat::shoot(0., twopi);
-  if(pi*(sqr(r1) + sqr(r2))/A >= z)
+  G4double phi = twopi*G4QuickRand();
+  G4double select = fSurfaceArea*G4QuickRand();
+  G4double z, rho;
+  if (select < pi*(sqr(r1) + sqr(r2)))
   {
-    G4double rho;
-    if(pi * sqr(r1) / A > z)
+    if(select < pi*sqr(r1))
     {
-      rho = r1 * std::sqrt(G4RandFlat::shoot(0., 1.));
-      return { rho * std::cos(phi), rho * std::sin(phi), -dz };
+      z = -dz;
+      rho = r1*std::sqrt(G4QuickRand());
     }
     else
     {
-      rho = r2 * std::sqrt(G4RandFlat::shoot(0., 1));
-      return { rho * std::cos(phi), rho * std::sin(phi), dz };
+      z = dz;
+      rho = r2*std::sqrt(G4QuickRand());
     }
   }
   else
   {
-    z = G4RandFlat::shoot(0., 1.)*2*dz - dz;
-    return { std::sqrt(z*k1 + k2)*std::cos(phi),
-             std::sqrt(z*k1 + k2)*std::sin(phi), z};
+    // rejection sampling
+    G4double mu_max = dz*k1 + k2 + 0.25*k1; // max surface element squared
+    for (auto i = 0; i < 10000; ++i)
+    {
+      z = dz*(2*G4QuickRand() - 1);
+      G4double mu = z*k1 + k2 + 0.25*k1; // surface element squared
+      if (mu_max*sqr(G4QuickRand()) <= mu) { break; }
+    }
+    rho = std::sqrt(z*k1 + k2);
   }
+  return { rho*std::cos(phi), rho*std::sin(phi), z };
 }
 
-/////////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
 //
 // Methods for visualisation
 //
