@@ -25,8 +25,8 @@
 //
 // G4VTwistSurface implementation
 //
-// 01-Aug-2002 - Kotoyo Hoshina (hoshina@hepburn.s.chiba-u.ac.jp), created.
-// 13-Nov-2003 - O.Link (Oliver.Link@cern.ch), Integration in Geant4
+// Author: Kotoyo Hoshina (Chiba University), 01.08.2002 - Created.
+//         Oliver Link (CERN), 13.11.2003 - Integration in Geant4
 //               from original version in Jupiter-2.5.02 application.
 // --------------------------------------------------------------------
 
@@ -412,7 +412,7 @@ G4double G4VTwistSurface::DistanceToIn(const G4ThreeVector& gp,
                   G4cout << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
                          << G4endl;
 #endif 
-                  if (tmpisvalid[k])  return kInfinity;
+                  if (tmpisvalid[k]) { return kInfinity; }
                   continue;
 
                //  
@@ -421,20 +421,20 @@ G4double G4VTwistSurface::DistanceToIn(const G4ThreeVector& gp,
                //
 
                }
-               else if (IsSameBoundary(this,areacode[i],
+               if (IsSameBoundary(this,areacode[i],
                                        neighbours[j], tmpareacode[k]))
                { 
                   // tmpxx[k] is same boundary (or corner) of xx.
                  
                   neighbournormal = neighbours[j]->GetNormal(tmpgxx[k], true);
-                  if (neighbournormal * gv < 0) isaccepted[j] = true;
+                  if (neighbournormal * gv < 0) { isaccepted[j] = true; }
                }
             } 
 
             // if nneighbours = 1, chabge isaccepted[1] before
             // exiting neighboursurface loop.  
 
-            if (nneighbours == 1) isaccepted[1] = true;
+            if (nneighbours == 1) { isaccepted[1] = true; }
 
          } // neighboursurface loop end
 
@@ -629,7 +629,7 @@ G4VTwistSurface::IsSameBoundary(G4VTwistSurface* surf1, G4int areacode1,
 
       return (corner1 - corner2).mag() < kCarTolerance;
    }
-   else if ((IsBoundary(areacode1, testbitmode) && (!iscorner[0])) &&
+   if ((IsBoundary(areacode1, testbitmode) && (!iscorner[0])) &&
             (IsBoundary(areacode2, testbitmode) && (!iscorner[1])))
    {
       // on boundary  
@@ -647,19 +647,16 @@ G4VTwistSurface::IsSameBoundary(G4VTwistSurface* surf1, G4int areacode1,
       return (x01 - x02).mag() < kCarTolerance
             && (d1 - d2).mag() < kCarTolerance;
    }
-   else
-   {
-      return false;
-   }
+   return false;
 }
 
 //=====================================================================
 //* GetBoundaryParameters ---------------------------------------------
 
 void G4VTwistSurface::GetBoundaryParameters(const G4int& areacode,
-                                             G4ThreeVector& d,
-                                             G4ThreeVector& x0,
-                                             G4int& boundarytype) const
+                                            G4ThreeVector& d,
+                                            G4ThreeVector& x0,
+                                            G4int& boundarytype) const
 {
    // areacode must be one of them:
    // sAxis0 & sAxisMin, sAxis0 & sAxisMax,
@@ -767,7 +764,7 @@ void G4VTwistSurface::SetCorner(G4int areacode,
 }
 
 //=====================================================================
-//* SetBoundaryAxis ---------------------------------------------------
+//* GetBoundaryAxis ---------------------------------------------------
 
 void G4VTwistSurface::GetBoundaryAxis(G4int areacode, EAxis axis[]) const
 {
@@ -897,39 +894,20 @@ G4int G4VTwistSurface::GetFace( G4int i, G4int j, G4int k,
   // this is the face mapping function
   // (i,j) -> face number
 
-  if ( iside == 0 ) {
-    return i * ( k - 1 ) + j ;
-  }
+  if ( iside == 0 ) { return i * ( k - 1 ) + j ; }
+  if ( iside == 1 ) { return (k-1)*(k-1) + i*(k-1) + j ; }
+  if ( iside == 2 ) { return 2*(k-1)*(k-1) + i*(k-1) + j ; }
+  if ( iside == 3 ) { return 2*(k-1)*(k-1) + (n-1)*(k-1) + i*(k-1) + j ; }
+  if ( iside == 4 ) { return 2*(k-1)*(k-1) + 2*(n-1)*(k-1) + i*(k-1) + j ; }
+  if ( iside == 5 ) { return 2*(k-1)*(k-1) + 3*(n-1)*(k-1) + i*(k-1) + j ; }
 
-  else if ( iside == 1 ) {
-    return (k-1)*(k-1) + i*(k-1) + j ;
-  }
-
-  else if ( iside == 2 ) {
-    return 2*(k-1)*(k-1) + i*(k-1) + j ;
-  }
-
-  else if ( iside == 3 ) {
-    return 2*(k-1)*(k-1) + (n-1)*(k-1) + i*(k-1) + j ;
-  }
-  
-  else if ( iside == 4 ) {
-    return 2*(k-1)*(k-1) + 2*(n-1)*(k-1) + i*(k-1) + j ;
-  }
-  
-  else if ( iside == 5 ) {
-    return 2*(k-1)*(k-1) + 3*(n-1)*(k-1) + i*(k-1) + j ;
-  }
-
-  else {
-    std::ostringstream message;
-    message << "Not correct side number: "
-            << GetName() << G4endl
-            << "iside is " << iside << " but should be "
-            << "0,1,2,3,4 or 5" << ".";
-    G4Exception("G4TwistSurface::G4GetFace()", "GeomSolids0002",
-                FatalException, message);
-  }
+  std::ostringstream message;
+  message << "Not correct side number: "
+          << GetName() << G4endl
+          << "iside is " << iside << " but should be "
+          << "0,1,2,3,4 or 5" << ".";
+  G4Exception("G4TwistSurface::G4GetFace()", "GeomSolids0002",
+              FatalException, message);
 
   return -1 ;  // wrong face
 }
@@ -950,66 +928,50 @@ G4int G4VTwistSurface::GetNode( G4int i, G4int j, G4int k,
     // n = k 
     return i * k + j ;
   }
-
   if ( iside == 1 )
   {
     // upper endcap is kxk squared. Shift by k*k
     // n = k 
     return  k*k + i*k + j ;
   }
-
-  else if ( iside == 2 )
+  if ( iside == 2 )
   {
     // front side.
-    if      ( i == 0 )   { return       j ;  }
-    else if ( i == n-1 ) { return k*k + j ;  } 
-    else                 { return 2*k*k + 4*(i-1)*(k-1) + j ; }
+    if ( i == 0 )   { return       j ;  }
+    if ( i == n-1 ) { return k*k + j ;  } 
+    return 2*k*k + 4*(i-1)*(k-1) + j ;
   }
-
-  else if ( iside == 3 )
+  if ( iside == 3 )
   {
     // right side
-    if      ( i == 0 )   { return       (j+1)*k - 1 ; } 
-    else if ( i == n-1 ) { return k*k + (j+1)*k - 1 ; }
-    else
-    {
-      return 2*k*k + 4*(i-1)*(k-1) + (k-1) + j ;
-    }
+    if ( i == 0 )   { return       (j+1)*k - 1 ; } 
+    if ( i == n-1 ) { return k*k + (j+1)*k - 1 ; }
+    return 2*k*k + 4*(i-1)*(k-1) + (k-1) + j ;
   }
-  else if ( iside == 4 )
+  if ( iside == 4 )
   {
     // back side
-    if      ( i == 0 )   { return   k*k - 1 - j ; }    // reversed order
-    else if ( i == n-1 ) { return 2*k*k - 1 - j ; }    // reversed order 
-    else
-    {
-      return 2*k*k + 4*(i-1)*(k-1) + 2*(k-1) + j ; // normal order
-    }
+    if ( i == 0 )   { return   k*k - 1 - j ; }    // reversed order
+    if ( i == n-1 ) { return 2*k*k - 1 - j ; }    // reversed order 
+    return 2*k*k + 4*(i-1)*(k-1) + 2*(k-1) + j ; // normal order
   }
-  else if ( iside == 5 )
+  if ( iside == 5 )
   {
     // left side 
-    if      ( i == 0 )   { return k*k   - (j+1)*k ; }  // reversed order
-    else if ( i == n-1)  { return 2*k*k - (j+1)*k ; }  // reverded order
-    else
-    {
-      if ( j == k-1 ) { return 2*k*k + 4*(i-1)*(k-1) ; } // special case
-      else
-      {
-        return 2*k*k + 4*(i-1)*(k-1) + 3*(k-1) + j ; // normal order
-      }
-    }
+    if      ( i == 0 ) { return k*k   - (j+1)*k ; }  // reversed order
+    if ( i == n-1) { return 2*k*k - (j+1)*k ; }  // reverded order
+    if ( j == k-1 ) { return 2*k*k + 4*(i-1)*(k-1) ; } // special case
+    return 2*k*k + 4*(i-1)*(k-1) + 3*(k-1) + j ; // normal order
   }
-  else
-  {
-    std::ostringstream message;
-    message << "Not correct side number: "
-            << GetName() << G4endl
-            << "iside is " << iside << " but should be "
-            << "0,1,2,3,4 or 5" << ".";
-    G4Exception("G4TwistSurface::G4GetNode()", "GeomSolids0002",
-                FatalException, message);
-  } 
+
+  std::ostringstream message;
+  message << "Not correct side number: "
+          << GetName() << G4endl
+          << "iside is " << iside << " but should be "
+          << "0,1,2,3,4 or 5" << ".";
+  G4Exception("G4TwistSurface::G4GetNode()", "GeomSolids0002",
+              FatalException, message);
+
   return -1 ;  // wrong node 
 } 
 
@@ -1055,62 +1017,57 @@ G4int G4VTwistSurface::GetEdgeVisibility( G4int i, G4int j, G4int k, G4int n,
   // check true edges
   if ( ( j>=1 && j<=k-3 ) )
   {
-    if ( i == 0 )  {        // signs (A):  ---+  
+    if ( i == 0 )        // signs (A):  ---+
+    {
       return ( number == 3 ) ? 1 : -1 ;
     }
     
-    else if ( i == n-2 ) {  // signs (C):  -+--
+    if ( i == n-2 )      // signs (C):  -+--
+    {
       return  ( number == 1 ) ? 1 : -1 ; 
     }
     
-    else
-    {
-      std::ostringstream message;
-      message << "Not correct face number: " << GetName() << " !";
-      G4Exception("G4TwistSurface::G4GetEdgeVisibility()",
-                  "GeomSolids0003", FatalException, message);
-    }
-  }
-  
-  if ( ( i>=1 && i<=n-3 ) )
-  {
-    if ( j == 0 )  {        // signs (D):  +---
-      return ( number == 0 ) ? 1 : -1 ;
-    }
-
-    else if ( j == k-2 ) {  // signs (B):  --+-
-      return  ( number == 2 ) ? 1 : -1 ; 
-    }
-
-    else
-    {
-      std::ostringstream message;
-      message << "Not correct face number: " << GetName() << " !";
-      G4Exception("G4TwistSurface::G4GetEdgeVisibility()",
-                  "GeomSolids0003", FatalException, message);
-    }
-  }
-  
-  // now the corners
-  if ( i == 0 && j == 0 ) {          // signs (a) : +--+
-    return ( number == 0 || number == 3 ) ? 1 : -1 ;
-  }
-  else if ( i == 0 && j == k-2 ) {   // signs (b) : --++
-    return ( number == 2 || number == 3 ) ? 1 : -1 ;
-  }
-  else if ( i == n-2 && j == k-2 ) { // signs (c) : -++-
-    return ( number == 1 || number == 2 ) ? 1 : -1 ;
-  }
-  else if ( i == n-2 && j == 0 ) {   // signs (d) : ++--
-    return ( number == 0 || number == 1 ) ? 1 : -1 ;
-  }
-  else
-  {
     std::ostringstream message;
     message << "Not correct face number: " << GetName() << " !";
     G4Exception("G4TwistSurface::G4GetEdgeVisibility()",
                 "GeomSolids0003", FatalException, message);
   }
+  
+  if ( ( i>=1 && i<=n-3 ) )
+  {
+    if ( j == 0 )    // signs (D):  +---
+    {
+      return ( number == 0 ) ? 1 : -1 ;
+    }
+
+    if ( j == k-2 )  // signs (B):  --+-
+    {
+      return  ( number == 2 ) ? 1 : -1 ; 
+    }
+    
+    std::ostringstream message;
+    message << "Not correct face number: " << GetName() << " !";
+    G4Exception("G4TwistSurface::G4GetEdgeVisibility()",
+                "GeomSolids0003", FatalException, message);
+  }
+  
+  // now the corners
+  if ( i == 0 && j == 0 )          // signs (a) : +--+
+  {
+    return ( number == 0 || number == 3 ) ? 1 : -1 ;
+  }
+  if ( i == 0 && j == k-2 )   // signs (b) : --++
+  {
+    return ( number == 2 || number == 3 ) ? 1 : -1 ;
+  }
+  if ( i == n-2 && j == k-2 ) // signs (c) : -++-
+  {
+    return ( number == 1 || number == 2 ) ? 1 : -1 ;
+  }
+  if ( i == n-2 && j == 0 )   // signs (d) : ++--
+  {
+    return ( number == 0 || number == 1 ) ? 1 : -1 ;
+  }
 
   std::ostringstream message;
   message << "Not correct face number: " << GetName() << " !";
@@ -1230,7 +1187,7 @@ G4VTwistSurface::CurrentStatus::ResetfDone(EValidate validate,
 {
   if (validate == fLastValidate && p != nullptr && *p == fLastp)
   {
-     if (v == nullptr || (*v == fLastv)) return;
+     if (v == nullptr || (*v == fLastv)) { return; }
   }         
   G4ThreeVector xx(kInfinity, kInfinity, kInfinity);
   for (size_t i=0; i<G4VSURFACENXX; ++i)
