@@ -25,7 +25,7 @@
 //
 // G4VTwistedFaceted implementation
 //
-// Author: 04-Nov-2004 - O.Link (Oliver.Link@cern.ch)
+// Author: Oliver Link (CERN), 27.10.2004 - Created
 // --------------------------------------------------------------------
 
 #include "G4VTwistedFaceted.hh"
@@ -39,19 +39,19 @@
 #include "G4ClippablePolygon.hh"
 #include "G4VPVParameterisation.hh"
 #include "G4GeometryTolerance.hh"
-#include "meshdefs.hh"
 
 #include "G4VGraphicsScene.hh"
 #include "G4Polyhedron.hh"
 #include "G4VisExtent.hh"
 
-#include "Randomize.hh"
+#include "G4QuickRand.hh"
 
 #include "G4AutoLock.hh"
 
 namespace
 {
   G4Mutex polyhedronMutex = G4MUTEX_INITIALIZER;
+  G4Mutex vtwMutex = G4MUTEX_INITIALIZER;
 }
 
 
@@ -392,13 +392,19 @@ EInside G4VTwistedFaceted::Inside(const G4ThreeVector& p) const
     if ( posy <= yMax - kCarTolerance*0.5
       && posy >= yMin + kCarTolerance*0.5 )
     {
-      if      (std::fabs(posz) <= fDz - kCarTolerance*0.5 ) tmpin = kInside ;
-      else if (std::fabs(posz) <= fDz + kCarTolerance*0.5 ) tmpin = kSurface ;
+      if      (std::fabs(posz) <= fDz - kCarTolerance*0.5 )
+      {
+        tmpin = kInside ;
+      }
+      else if (std::fabs(posz) <= fDz + kCarTolerance*0.5 )
+      {
+        tmpin = kSurface ;
+      }
     }
     else if ( posy <= yMax + kCarTolerance*0.5
            && posy >= yMin - kCarTolerance*0.5 )
     {
-      if (std::fabs(posz) <= fDz + kCarTolerance*0.5 ) tmpin = kSurface ;
+      if (std::fabs(posz) <= fDz + kCarTolerance*0.5 ) { tmpin = kSurface ; }
     }
   }
   else if ( posx <= xMax + kCarTolerance*0.5
@@ -407,7 +413,7 @@ EInside G4VTwistedFaceted::Inside(const G4ThreeVector& p) const
     if ( posy <= yMax + kCarTolerance*0.5
       && posy >= yMin - kCarTolerance*0.5 )
     {
-      if (std::fabs(posz) <= fDz + kCarTolerance*0.5) tmpin = kSurface ;
+      if (std::fabs(posz) <= fDz + kCarTolerance*0.5) { tmpin = kSurface ; }
     }
   }
 
@@ -887,19 +893,6 @@ void G4VTwistedFaceted::CreateSurfaces()
 
 }
 
-//=====================================================================
-//* GetCubicVolume ----------------------------------------------------
-
-G4double G4VTwistedFaceted::GetCubicVolume()
-{
-  if(fCubicVolume == 0.)
-  {
-    fCubicVolume = ((fDx1 + fDx2 + fDx3 + fDx4)*(fDy1 + fDy2) +
-                    (fDx4 + fDx3 - fDx2 - fDx1)*(fDy2 - fDy1)/3)*fDz;
-  }
-  return fCubicVolume;
-}
-
 //=====================================================================
 //* GetLateralFaceArea ------------------------------------------------
 
@@ -975,13 +968,29 @@ G4VTwistedFaceted::GetLateralFaceArea(const G4TwoVector& p1,
   return area*dt;
 }
 
+//=====================================================================
+//* GetCubicVolume ----------------------------------------------------
+
+G4double G4VTwistedFaceted::GetCubicVolume()
+{
+  if(fCubicVolume == 0)
+  {
+    G4AutoLock l(&vtwMutex);
+    fCubicVolume = ((fDx1 + fDx2 + fDx3 + fDx4)*(fDy1 + fDy2) +
+                    (fDx4 + fDx3 - fDx2 - fDx1)*(fDy2 - fDy1)/3)*fDz;
+    l.unlock();
+  }
+  return fCubicVolume;
+}
+
 //=====================================================================
 //* GetSurfaceArea ----------------------------------------------------
 
 G4double G4VTwistedFaceted::GetSurfaceArea()
 {
-  if (fSurfaceArea == 0.)
+  if (fSurfaceArea == 0)
   {
+    G4AutoLock l(&vtwMutex);
     G4TwoVector vv[8];
     vv[0] = G4TwoVector(-fDx1 - fDy1*fTAlph,-fDy1);
     vv[1] = G4TwoVector( fDx1 - fDy1*fTAlph,-fDy1);
@@ -996,6 +1005,7 @@ G4double G4VTwistedFaceted::GetSurfaceArea()
       GetLateralFaceArea(vv[1], vv[3], vv[5], vv[7]) +
       GetLateralFaceArea(vv[3], vv[2], vv[7], vv[6]) +
       GetLateralFaceArea(vv[2], vv[0], vv[6], vv[4]);
+    l.unlock();
   }
   return fSurfaceArea;
 }
@@ -1030,32 +1040,13 @@ G4Polyhedron* G4VTwistedFaceted::GetPolyhedron() const
 }
 
 
-//=====================================================================
-//* GetPointInSolid ---------------------------------------------------
-
-G4ThreeVector G4VTwistedFaceted::GetPointInSolid(G4double z) const
-{
-
-
-  // this routine is only used for a test
-  // can be deleted ...
-
-  if ( z == fDz ) z -= 0.1*fDz ;
-  if ( z == -fDz ) z += 0.1*fDz ;
-
-  G4double phi = z/(2*fDz)*fPhiTwist ;
-
-  return { fdeltaX * phi/fPhiTwist, fdeltaY * phi/fPhiTwist, z };
-}
-
-
 //=====================================================================
 //* GetPointOnSurface -------------------------------------------------
 
 G4ThreeVector G4VTwistedFaceted::GetPointOnSurface() const
 {
 
-  G4double phi = G4RandFlat::shoot(-fPhiTwist/2.,fPhiTwist/2.);
+  G4double phi = fPhiTwist*(G4QuickRand() - 0.5);
   G4double u , umin, umax ;  //  variable for twisted surfaces
   G4double y  ;              //  variable for flat surface (top and bottom)
 
@@ -1080,60 +1071,49 @@ G4ThreeVector G4VTwistedFaceted::GetPointOnSurface() const
   G4cout << "Surface Upper   = " << a6 << G4endl ;
 #endif 
 
-  G4double chose = G4RandFlat::shoot(0.,a1 + a2 + a3 + a4 + a5 + a6) ;
+  G4double chose = (a1 + a2 + a3 + a4 + a5 + a6)*G4QuickRand() ;
 
   if(chose < a1)
   {
     umin = fSide0->GetBoundaryMin(phi) ;
     umax = fSide0->GetBoundaryMax(phi) ;
-    u = G4RandFlat::shoot(umin,umax) ;
-
-    return  fSide0->SurfacePoint(phi, u, true) ;   // point on 0deg surface
+    u = umin + (umax - umin)*G4QuickRand();
+    return  fSide0->SurfacePoint(phi, u, true) ;   // point on 0 deg surface
   }
-
-  else if( (chose >= a1) && (chose < a1 + a2 ) )
+  if( (chose >= a1) && (chose < a1 + a2 ) )
   {
     umin = fSide90->GetBoundaryMin(phi) ;
     umax = fSide90->GetBoundaryMax(phi) ;
-    
-    u = G4RandFlat::shoot(umin,umax) ;
-
-    return fSide90->SurfacePoint(phi, u, true);   // point on 90deg surface
+    u = umin + (umax - umin)*G4QuickRand();
+    return fSide90->SurfacePoint(phi, u, true);   // point on 90 deg surface
   }
-  else if( (chose >= a1 + a2 ) && (chose < a1 + a2 + a3 ) )
+  if( (chose >= a1 + a2 ) && (chose < a1 + a2 + a3 ) )
   {
     umin = fSide180->GetBoundaryMin(phi) ;
     umax = fSide180->GetBoundaryMax(phi) ;
-    u = G4RandFlat::shoot(umin,umax) ;
-
+    u = umin + (umax - umin)*G4QuickRand();
     return fSide180->SurfacePoint(phi, u, true); // point on 180 deg surface
   }
-  else if( (chose >= a1 + a2 + a3  ) && (chose < a1 + a2 + a3 + a4  ) )
+  if( (chose >= a1 + a2 + a3  ) && (chose < a1 + a2 + a3 + a4  ) )
   {
     umin = fSide270->GetBoundaryMin(phi) ;
     umax = fSide270->GetBoundaryMax(phi) ;
-    u = G4RandFlat::shoot(umin,umax) ;
+    u = umin + (umax - umin)*G4QuickRand();
     return fSide270->SurfacePoint(phi, u, true); // point on 270 deg surface
   }
-  else if( (chose >= a1 + a2 + a3 + a4  ) && (chose < a1 + a2 + a3 + a4 + a5 ) )
+  if( (chose >= a1 + a2 + a3 + a4  ) && (chose < a1 + a2 + a3 + a4 + a5 ) )
   {
-    y = G4RandFlat::shoot(-fDy1,fDy1) ;
+    y = fDy1*(2.*G4QuickRand() - 1.);
     umin = fLowerEndcap->GetBoundaryMin(y) ;
     umax = fLowerEndcap->GetBoundaryMax(y) ;
-    u = G4RandFlat::shoot(umin,umax) ;
-
+    u = umin + (umax - umin)*G4QuickRand();
     return fLowerEndcap->SurfacePoint(u,y,true); // point on lower endcap
   }
-  else
-  {
-    y = G4RandFlat::shoot(-fDy2,fDy2) ;
-    umin = fUpperEndcap->GetBoundaryMin(y) ;
-    umax = fUpperEndcap->GetBoundaryMax(y) ;
-    u = G4RandFlat::shoot(umin,umax) ;
-
-    return fUpperEndcap->SurfacePoint(u,y,true) ; // point on upper endcap
-
-  }
+  y = fDy2*(2.*G4QuickRand() - 1.);
+  umin = fUpperEndcap->GetBoundaryMin(y) ;
+  umax = fUpperEndcap->GetBoundaryMax(y) ;
+  u = umin + (umax - umin)*G4QuickRand();
+  return fUpperEndcap->SurfacePoint(u,y,true); // point on upper endcap
 }
 
 
