@@ -25,7 +25,7 @@
 //
 // G4Voxelizer implementation
 //
-// 19.10.12 Marek Gayer, created
+// Author: Marek Gayer (CERN), 19.10.2012 - Created
 // --------------------------------------------------------------------
 
 #include <iostream>
@@ -39,16 +39,13 @@
 #include "G4CSGSolid.hh"
 #include "G4GeometryTolerance.hh"
 #include "G4Orb.hh"
-#include "G4PhysicalConstants.hh"
 #include "G4SolidStore.hh"
 #include "G4Types.hh"
 #include "G4Voxelizer.hh"
-#include "Randomize.hh"
-#include "geomdefs.hh"
 
 using namespace std;
 
-G4ThreadLocal G4int G4Voxelizer::fDefaultVoxelsCount = -1;
+G4int G4Voxelizer::fDefaultVoxelsCount = -1;
 
 //______________________________________________________________________________
 G4Voxelizer::G4Voxelizer()
@@ -63,9 +60,6 @@ G4Voxelizer::G4Voxelizer()
   G4SolidStore::GetInstance()->DeRegister(&fBoundingBox);
 }
 
-//______________________________________________________________________________
-G4Voxelizer::~G4Voxelizer() = default;
-
 //______________________________________________________________________________
 void G4Voxelizer::BuildEmpty()
 {
@@ -75,7 +69,10 @@ void G4Voxelizer::BuildEmpty()
   std::vector<G4int> xyz(3), max(3), candidates(fTotalCandidates);
   const std::vector<G4int> empty(0);
 
-  for (auto i = 0; i <= 2; ++i) max[i] = (G4int)fBoundaries[i].size();
+  for (auto i = 0; i <= 2; ++i)
+  {
+    max[i] = (G4int)fBoundaries[i].size();
+  }
   unsigned int size = max[0] * max[1] * max[2];
 
   fEmpty.Clear();
@@ -333,7 +330,7 @@ void G4Voxelizer::DisplayBoundaries(std::vector<G4double> &boundaries)
   for(std::size_t i = 0; i < count; ++i)
   {
     G4cout << setw(10) << setiosflags(ios::fixed) << boundaries[i];
-    if(i != count-1) G4cout << "-> ";
+    if(i != count-1) { G4cout << "-> "; }
   }
   G4cout << "|" << G4endl << "Number of boundaries: " << count << G4endl;
   G4cout.precision(oldprec);
@@ -368,7 +365,10 @@ void G4Voxelizer::BuildBitmasks(std::vector<G4double> boundaries[],
     std::vector<G4int>& candidatesCount = fCandidatesCounts[k];
     candidatesCount.resize(voxelsCount);
 
-    for(G4int i = 0 ; i < voxelsCount; ++i) { candidatesCount[i] = 0; }
+    for(G4int i = 0 ; i < voxelsCount; ++i)
+    {
+      candidatesCount[i] = 0;
+    }
 
     // Loop on the nodes, number of slices per axis
     //
@@ -533,8 +533,7 @@ void G4Voxelizer::BuildReduceVoxels(std::vector<G4double> boundaries[],
       while ((currentCount = (G4int)voxelSet.size()) > 2) 
       {
         G4double currentRatio = 1 - (G4double) count / max;
-        if ((currentRatio <= reduction) && (currentCount <= 1000))
-          break;
+        if ((currentRatio <= reduction) && (currentCount <= 1000)) { break; }
         const G4int pos = *voxelSet.begin();
         mergings.push_back(pos + 1);
 
@@ -546,21 +545,27 @@ void G4Voxelizer::BuildReduceVoxels(std::vector<G4double> boundaries[],
           ;// k = k;
         }
         if (voxel.next != max - 1)
+        {
           if (voxelSet.erase(voxel.next) != 1)
           {
             ;// k = k;
           }
+        }
         if (voxel.previous != -1)
+        {
           if (voxelSet.erase(voxel.previous) != 1)
           {
             ;// k = k;
           }
+        }
         nextVoxel.count += voxel.count;
         voxel.count = 0;
         nextVoxel.previous = voxel.previous;
 
         if (voxel.next != max - 1)
+        {
           voxelSet.insert(voxel.next);
+        }
 
         if (voxel.previous != -1)
         {
@@ -593,66 +598,6 @@ void G4Voxelizer::BuildReduceVoxels(std::vector<G4double> boundaries[],
       }
       boundaries[k] = std::move(reducedBoundary);
     }
-/*
-    G4int count = 0;
-    while (true)    // Loop checking, 13.08.2015, G.Cosmo
-    {
-      G4double reduction = reductionRatio[k];
-      if (reduction == 0)
-        break;
-      G4int currentCount = voxelSet.size();
-      if (currentCount <= 2)
-        break;
-      G4double currentRatio = 1 - (G4double) count / max;
-      if (currentRatio <= reduction && currentCount <= 1000)
-        break;
-      const G4int pos = *voxelSet.begin();
-      mergings.push_back(pos);
-
-      G4VoxelInfo &voxel = voxels[pos];
-      G4VoxelInfo &nextVoxel = voxels[voxel.next];
-
-      voxelSet.erase(pos);
-      if (voxel.next != max - 1) { voxelSet.erase(voxel.next); }
-      if (voxel.previous != -1)  { voxelSet.erase(voxel.previous); }
-
-      nextVoxel.count += voxel.count;
-      voxel.count = 0;
-      nextVoxel.previous = voxel.previous;
-
-      if (voxel.next != max - 1)
-        voxelSet.insert(voxel.next);
-
-      if (voxel.previous != -1)
-      {
-        voxels[voxel.previous].next = voxel.next;
-        voxelSet.insert(voxel.previous);
-      }
-      ++count;
-    }
-
-    if (mergings.size())
-    {
-      std::sort(mergings.begin(), mergings.end());
-
-      std::vector<G4double> &boundary = boundaries[k];
-      std::vector<G4double> reducedBoundary(boundary.size() - mergings.size());
-      G4int skip = mergings[0] + 1, cur = 0, i = 0;
-      max = boundary.size();
-      for (G4int j = 0; j < max; ++j)
-      {
-        if (j != skip)
-        {
-          reducedBoundary[cur++] = boundary[j];
-        }
-        else
-        {
-          if (++i < (G4int)mergings.size())  { skip = mergings[i] + 1; }
-        }
-      }
-      boundaries[k] = reducedBoundary;
-    }
-*/
   }
 }
 
@@ -665,15 +610,17 @@ void G4Voxelizer::BuildReduceVoxels2(std::vector<G4double> boundaries[],
     std::vector<G4int> &candidatesCount = fCandidatesCounts[k];
     auto max = (G4int)candidatesCount.size();
     G4int total = 0;
-    for (G4int i = 0; i < max; ++i) total += candidatesCount[i];
+    for (G4int i = 0; i < max; ++i)
+    {
+      total += candidatesCount[i];
+    }
 
     G4double reduction = reductionRatio[k];
-    if (reduction == 0)
-      break;
+    if (reduction == 0) { break; }
 
     G4int destination = (G4int) (reduction * max) + 1;
-    if (destination > 1000) destination = 1000;
-    if (destination < 2) destination = 2;
+    if (destination > 1000) { destination = 1000; }
+    if (destination < 2) { destination = 2; }
     G4double average = ((G4double)total / max) / reduction;
 
     std::vector<G4int> mergings;
@@ -690,8 +637,7 @@ void G4Voxelizer::BuildReduceVoxels2(std::vector<G4double> boundaries[],
         G4double val = boundary[i];
         reducedBoundary[cur] = val;
         ++cur;
-        if (cur == destination)
-          break;
+        if (cur == destination) { break; }
       }
     }
     reducedBoundary[destination-1] = boundary[max];
@@ -722,7 +668,10 @@ void G4Voxelizer::CreateMiniVoxels(std::vector<G4double> boundaries[],
                                    G4SurfBits bitmasks[])
 {
   std::vector<G4int> voxel(3), maxVoxels(3);
-  for (auto i = 0; i <= 2; ++i) maxVoxels[i] = (G4int)boundaries[i].size();
+  for (auto i = 0; i <= 2; ++i)
+  {
+    maxVoxels[i] = (G4int)boundaries[i].size();
+  }
 
   G4ThreeVector point;
   for (voxel[2] = 0; voxel[2] < maxVoxels[2] - 1; ++voxel[2])
@@ -764,7 +713,7 @@ void G4Voxelizer::Voxelize(std::vector<G4VFacet*>& facets)
   {
     for (const auto & facet : facets)
     { 
-      if (facet->GetNumberOfVertices() > 3) ++size;
+      if (facet->GetNumberOfVertices() > 3) { ++size; }
     }
   }
 
@@ -791,7 +740,7 @@ void G4Voxelizer::Voxelize(std::vector<G4VFacet*>& facets)
     if (maxVoxels < 0 && reductionRatio == G4ThreeVector())
     {
       maxVoxels = fTotalCandidates;
-      if (fTotalCandidates > 1000000) maxVoxels = 1000000;
+      if (fTotalCandidates > 1000000) { maxVoxels = 1000000; }
     }
 
     SetReductionRatio(maxVoxels, reductionRatio);
@@ -825,7 +774,10 @@ void G4Voxelizer::Voxelize(std::vector<G4VFacet*>& facets)
 
     std::vector<G4double> miniBoundaries[3];
 
-    for (auto i = 0; i <= 2; ++i)  { miniBoundaries[i] = fBoundaries[i]; }
+    for (auto i = 0; i <= 2; ++i)
+    {
+      miniBoundaries[i] = fBoundaries[i];
+    }
 
     G4int voxelsCountMini = (fCountOfVoxels >= 1000)
                           ? 100 : G4int(fCountOfVoxels / 10);
@@ -893,13 +845,16 @@ void G4Voxelizer::GetCandidatesVoxel(std::vector<G4int>& voxels)
   std::vector<G4int> candidates;
   G4int count = GetCandidatesVoxelArray(voxels, candidates);
   G4cout << "[ ";
-  for (G4int i = 0; i < count; ++i) G4cout << candidates[i];
+  for (G4int i = 0; i < count; ++i)
+  {
+    G4cout << candidates[i];
+  }
   G4cout << "]  " << G4endl;
 }
 
 //______________________________________________________________________________
 void G4Voxelizer::FindComponentsFastest(unsigned int mask,
-                                         std::vector<G4int>& list, G4int i)
+                                         std::vector<G4int>& list, G4int i) const
 {
   for (G4int byte = 0; byte < (G4int) (sizeof(unsigned int)); ++byte)
   {
@@ -909,7 +864,7 @@ void G4Voxelizer::FindComponentsFastest(unsigned int mask,
       {
         if ((maskByte & 1) != 0) 
           { list.push_back(8*(sizeof(unsigned int)*i+ byte) + bit); }
-        if ((maskByte >>= 1) == 0) break;
+        if ((maskByte >>= 1) == 0) { break; }
       }
     }
     mask >>= 8;
@@ -948,14 +903,14 @@ void G4Voxelizer::TransformLimits(G4ThreeVector& min, G4ThreeVector& max,
     G4ThreeVector current = GetGlobalPoint(transformation, vertices[i]);
 
     // If need be, replacement of the min & max values:
-    if (current.x() > max.x()) max.setX(current.x());
-    if (current.x() < min.x()) min.setX(current.x());
+    if (current.x() > max.x()) { max.setX(current.x()); }
+    if (current.x() < min.x()) { min.setX(current.x()); }
 
-    if (current.y() > max.y()) max.setY(current.y());
-    if (current.y() < min.y()) min.setY(current.y());
+    if (current.y() > max.y()) { max.setY(current.y()); }
+    if (current.y() < min.y()) { min.setY(current.y()); }
 
-    if (current.z() > max.z()) max.setZ(current.z());
-    if (current.z() < min.z()) min.setZ(current.z());
+    if (current.z() > max.z()) { max.setZ(current.z()); }
+    if (current.z() < min.z()) { min.setZ(current.z()); }
   }
 }
 
@@ -969,8 +924,10 @@ G4int G4Voxelizer::GetCandidatesVoxelArray(const G4ThreeVector &point,
 
   for (auto i = 0; i <= 2; ++i)
   {
-    if(point[i] < fBoundaries[i].front() || point[i] >= fBoundaries[i].back()) 
+    if(point[i] < fBoundaries[i].front() || point[i] >= fBoundaries[i].back())
+    { 
       return 0;
+    }
   }
 
   if (fTotalCandidates == 1)
@@ -978,103 +935,67 @@ G4int G4Voxelizer::GetCandidatesVoxelArray(const G4ThreeVector &point,
     list.push_back(0);
     return 1;
   } 
+  
+  if (fNPerSlice == 1)
+  {
+    unsigned int mask = 0xFFffFFff;
+    G4int slice;
+    if (fBoundaries[0].size() > 2)
+    {
+      slice = BinarySearch(fBoundaries[0], point.x());
+      if ((mask = ((unsigned int*) fBitmasks[0].fAllBits)[slice]) == 0u)
+      {
+        return 0;
+      }
+    }
+    if (fBoundaries[1].size() > 2)
+    {
+      slice = BinarySearch(fBoundaries[1], point.y());
+      if ((mask &= ((unsigned int*) fBitmasks[1].fAllBits)[slice]) == 0u)
+      {
+        return 0;
+      }
+    }
+    if (fBoundaries[2].size() > 2)
+    {
+      slice = BinarySearch(fBoundaries[2], point.z());
+      if ((mask &= ((unsigned int*) fBitmasks[2].fAllBits)[slice]) == 0u)
+      {
+        return 0;
+      }
+    }
+    if ((crossed != nullptr) && ((mask &= ~((unsigned int*)crossed->fAllBits)[0]) == 0u))
+    {
+      return 0;
+    }
+    FindComponentsFastest(mask, list, 0);
+  }
   else
   {
-    if (fNPerSlice == 1)
+    unsigned int* masks[3], mask; // masks for X,Y,Z axis
+    for (auto i = 0; i <= 2; ++i)
     {
-      unsigned int mask = 0xFFffFFff;
-      G4int slice;
-      if (fBoundaries[0].size() > 2)
-      {
-        slice = BinarySearch(fBoundaries[0], point.x());
-        if ((mask = ((unsigned int*) fBitmasks[0].fAllBits)[slice]) == 0u)
-          return 0;
-      }
-      if (fBoundaries[1].size() > 2)
-      {
-        slice = BinarySearch(fBoundaries[1], point.y());
-        if ((mask &= ((unsigned int*) fBitmasks[1].fAllBits)[slice]) == 0u)
-          return 0;
-      }
-      if (fBoundaries[2].size() > 2)
-      {
-        slice = BinarySearch(fBoundaries[2], point.z());
-        if ((mask &= ((unsigned int*) fBitmasks[2].fAllBits)[slice]) == 0u)
-          return 0;
-      }
-      if ((crossed != nullptr) && ((mask &= ~((unsigned int*)crossed->fAllBits)[0]) == 0u))
-        return 0;
-
-      FindComponentsFastest(mask, list, 0);
+      G4int slice = BinarySearch(fBoundaries[i], point[i]);
+      masks[i] = ((unsigned int*) fBitmasks[i].fAllBits)
+               + slice * fNPerSlice;
     }
-    else
+    unsigned int* maskCrossed = crossed != nullptr
+                              ? (unsigned int*)crossed->fAllBits : nullptr;
+
+    for (G4int i = 0 ; i < fNPerSlice; ++i)
     {
-      unsigned int* masks[3], mask; // masks for X,Y,Z axis
-      for (auto i = 0; i <= 2; ++i)
+      // Logic "and" of the masks along the 3 axes x, y, z:
+      // removing "if (!" and ") continue" => slightly slower
+      //
+      if ((mask = masks[0][i]) == 0u) { continue; }
+      if ((mask &= masks[1][i]) == 0u) { continue; }
+      if ((mask &= masks[2][i]) == 0u) { continue; }
+      if ((maskCrossed != nullptr) && ((mask &= ~maskCrossed[i]) == 0u))
       {
-        G4int slice = BinarySearch(fBoundaries[i], point[i]);
-        masks[i] = ((unsigned int*) fBitmasks[i].fAllBits)
-                 + slice * fNPerSlice;
-      }
-      unsigned int* maskCrossed = crossed != nullptr
-                                ? (unsigned int*)crossed->fAllBits : nullptr;
-
-      for (G4int i = 0 ; i < fNPerSlice; ++i)
-      {
-        // Logic "and" of the masks along the 3 axes x, y, z:
-        // removing "if (!" and ") continue" => slightly slower
-        //
-        if ((mask = masks[0][i]) == 0u) continue;
-        if ((mask &= masks[1][i]) == 0u) continue;
-        if ((mask &= masks[2][i]) == 0u) continue;
-        if ((maskCrossed != nullptr) && ((mask &= ~maskCrossed[i]) == 0u)) continue;
-
-        FindComponentsFastest(mask, list, i);
+        continue;
       }
+      FindComponentsFastest(mask, list, i);
     }
-/*
-    if (fNPerSlice == 1)
-    {
-      unsigned int mask;
-      G4int slice = BinarySearch(fBoundaries[0], point.x()); 
-      if (!(mask = ((unsigned int *) fBitmasks[0].fAllBits)[slice]
-      )) return 0;
-      slice = BinarySearch(fBoundaries[1], point.y());
-      if (!(mask &= ((unsigned int *) fBitmasks[1].fAllBits)[slice]
-      )) return 0;
-      slice = BinarySearch(fBoundaries[2], point.z());
-      if (!(mask &= ((unsigned int *) fBitmasks[2].fAllBits)[slice]
-      )) return 0;
-      if (crossed && (!(mask &= ~((unsigned int *)crossed->fAllBits)[0])))
-        return 0;
-
-      FindComponentsFastest(mask, list, 0);
-    }
-    else
-    {
-      unsigned int *masks[3], mask; // masks for X,Y,Z axis
-      for (auto i = 0; i <= 2; ++i)
-      {
-        G4int slice = BinarySearch(fBoundaries[i], point[i]); 
-        masks[i] = ((unsigned int *) fBitmasks[i].fAllBits) + slice*fNPerSlice;
-      }
-      unsigned int *maskCrossed =
-        crossed ? (unsigned int *)crossed->fAllBits : 0;
-
-      for (G4int i = 0 ; i < fNPerSlice; ++i)
-      {
-        // Logic "and" of the masks along the 3 axes x, y, z:
-        // removing "if (!" and ") continue" => slightly slower
-        //
-        if (!(mask = masks[0][i])) continue;
-        if (!(mask &= masks[1][i])) continue;
-        if (!(mask &= masks[2][i])) continue;
-        if (maskCrossed && !(mask &= ~maskCrossed[i])) continue;
-
-        FindComponentsFastest(mask, list, i);
-      }
-    }
-*/
   }
   return (G4int)list.size();
 }
@@ -1093,45 +1014,52 @@ G4Voxelizer::GetCandidatesVoxelArray(const std::vector<G4int>& voxels,
     list.push_back(0);
     return 1;
   }
+  
+  if (fNPerSlice == 1)
+  {
+    unsigned int mask;
+    if ((mask = ((unsigned int *) bitmasks[0].fAllBits)[voxels[0]]) == 0u)
+    {
+      return 0;
+    }
+    if ((mask &= ((unsigned int *) bitmasks[1].fAllBits)[voxels[1]]) == 0u)
+    {
+      return 0;
+    }
+    if ((mask &= ((unsigned int *) bitmasks[2].fAllBits)[voxels[2]]) == 0u)
+    {
+      return 0;
+    }
+    if ((crossed != nullptr) && ((mask &= ~((unsigned int *)crossed->fAllBits)[0]) == 0u))
+    {
+      return 0;
+    }
+    FindComponentsFastest(mask, list, 0);
+  }
   else
   {
-    if (fNPerSlice == 1)
+    unsigned int *masks[3], mask; // masks for X,Y,Z axis
+    for (auto i = 0; i <= 2; ++i)
     {
-      unsigned int mask;
-      if ((mask = ((unsigned int *) bitmasks[0].fAllBits)[voxels[0]]) == 0u)
-        return 0;
-      if ((mask &= ((unsigned int *) bitmasks[1].fAllBits)[voxels[1]]) == 0u)
-        return 0;
-      if ((mask &= ((unsigned int *) bitmasks[2].fAllBits)[voxels[2]]) == 0u)
-        return 0;
-      if ((crossed != nullptr) && ((mask &= ~((unsigned int *)crossed->fAllBits)[0]) == 0u))
-        return 0;
-
-      FindComponentsFastest(mask, list, 0);
+      masks[i] = ((unsigned int *) bitmasks[i].fAllBits)
+               + voxels[i]*fNPerSlice;
     }
-    else
+    unsigned int *maskCrossed = crossed != nullptr
+                              ? (unsigned int *)crossed->fAllBits : nullptr;
+
+    for (G4int i = 0 ; i < fNPerSlice; ++i)
     {
-      unsigned int *masks[3], mask; // masks for X,Y,Z axis
-      for (auto i = 0; i <= 2; ++i)
+      // Logic "and" of the masks along the 3 axes x, y, z:
+      // removing "if (!" and ") continue" => slightly slower
+      //
+      if ((mask = masks[0][i]) == 0u) { continue; }
+      if ((mask &= masks[1][i]) == 0u) { continue; }
+      if ((mask &= masks[2][i]) == 0u) { continue; }
+      if ((maskCrossed != nullptr) && ((mask &= ~maskCrossed[i]) == 0u))
       {
-        masks[i] = ((unsigned int *) bitmasks[i].fAllBits)
-                 + voxels[i]*fNPerSlice;
-      }
-      unsigned int *maskCrossed = crossed != nullptr
-                                ? (unsigned int *)crossed->fAllBits : nullptr;
-
-      for (G4int i = 0 ; i < fNPerSlice; ++i)
-      {
-        // Logic "and" of the masks along the 3 axes x, y, z:
-        // removing "if (!" and ") continue" => slightly slower
-        //
-        if ((mask = masks[0][i]) == 0u) continue;
-        if ((mask &= masks[1][i]) == 0u) continue;
-        if ((mask &= masks[2][i]) == 0u) continue;
-        if ((maskCrossed != nullptr) && ((mask &= ~maskCrossed[i]) == 0u)) continue;
-
-        FindComponentsFastest(mask, list, i);
+        continue;
       }
+      FindComponentsFastest(mask, list, i);
     }
   }
   return (G4int)list.size();
@@ -1153,7 +1081,9 @@ G4bool G4Voxelizer::Contains(const G4ThreeVector& point) const
   for (auto i = 0; i < 3; ++i)
   {
     if (point[i] < fBoundaries[i].front() || point[i] > fBoundaries[i].back())
+    {
       return false;
+    }
   }
   return true;
 }
@@ -1180,7 +1110,7 @@ G4Voxelizer::DistanceToBoundingBox(const G4ThreeVector& point) const
 //______________________________________________________________________________
 G4double
 G4Voxelizer::MinDistanceToBox (const G4ThreeVector& aPoint,
-                               const G4ThreeVector& f)
+                               const G4ThreeVector& f) const
 {
   // Estimates the isotropic safety from a point outside the current solid to
   // any of its surfaces. The algorithm may be accurate or should provide a
@@ -1189,17 +1119,17 @@ G4Voxelizer::MinDistanceToBox (const G4ThreeVector& aPoint,
   G4double safe, safx, safy, safz;
   safe = safx = -f.x() + std::abs(aPoint.x());
   safy = -f.y() + std::abs(aPoint.y());
-  if ( safy > safe ) safe = safy;
+  if ( safy > safe ) { safe = safy; }
   safz = -f.z() + std::abs(aPoint.z());
-  if ( safz > safe ) safe = safz;
-  if (safe < 0.0) return 0.0; // point is inside
+  if ( safz > safe ) { safe = safz; }
+  if (safe < 0.0) { return 0.0; } // point is inside
 
   G4double safsq = 0.0;
   G4int count = 0;
   if ( safx > 0 ) { safsq += safx*safx; ++count; }
   if ( safy > 0 ) { safsq += safy*safy; ++count; }
   if ( safz > 0 ) { safsq += safz*safz; ++count; }
-  if (count == 1) return safe;
+  if (count == 1) { return safe; }
   return std::sqrt(safsq);
 }
 
@@ -1224,8 +1154,7 @@ G4Voxelizer::DistanceToNext(const G4ThreeVector& point,
     }
     else
     {
-      if (direction[i] > -1e-10)
-        continue;
+      if (direction[i] > -1e-10) { continue; }
     }
     G4double dif = boundary[index] - point[i];
     G4double distance = dif / direction[i];
@@ -1245,46 +1174,18 @@ G4Voxelizer::DistanceToNext(const G4ThreeVector& point,
     if (direction[cur] > 0)
     {
       if (++curVoxel[cur] >= (G4int) fBoundaries[cur].size() - 1)
+      {
         shift = kInfinity;
+      }
     }
     else
     {
       if (--curVoxel[cur] < 0)
-        shift = kInfinity;
-    }
-  }
-
-/*
-  for (auto i = 0; i <= 2; ++i)
-  {
-    // Looking for the next voxels on the considered direction X,Y,Z axis
-    //
-    const std::vector<G4double> &boundary = fBoundaries[i];
-    G4int cur = curVoxel[i];
-    if(direction[i] >= 1e-10)
-    {
-        if (boundary[++cur] - point[i] < fTolerance) // make sure shift would
-        if (++cur >= (G4int) boundary.size())      // be non-zero
-          continue;
-    }
-    else 
-    {
-      if(direction[i] <= -1e-10) 
       {
-        if (point[i] - boundary[cur] < fTolerance) // make sure shift would
-          if (--cur < 0)                           // be non-zero
-            continue;
+        shift = kInfinity;
       }
-      else 
-        continue;
     }
-    G4double dif = boundary[cur] - point[i];
-    G4double distance = dif / direction[i];
-
-    if (shift > distance) 
-      shift = distance;
   }
-*/
   return shift;
 }
 
@@ -1301,15 +1202,17 @@ G4Voxelizer::UpdateCurrentVoxel(const G4ThreeVector& point,
 
     if (direction[i] > 0) 
     {
-      if (point[i] >= boundary[++index]) 
-        if (++curVoxel[i] >= (G4int) boundary.size() - 1)
-          return false;
+      if (point[i] >= boundary[++index])
+      { 
+        if (++curVoxel[i] >= (G4int) boundary.size() - 1) { return false; }
+      }
     }
     else
     {
-      if (point[i] < boundary[index]) 
-        if (--curVoxel[i] < 0) 
-          return false;
+      if (point[i] < boundary[index])
+      { 
+        if (--curVoxel[i] < 0) { return false; }
+      }
     }
 #ifdef G4SPECSDEBUG
     G4int indexOK = BinarySearch(boundary, point[i]);
