@@ -91,7 +91,7 @@ G4DNARuddIonisationExtendedModel::G4DNARuddIonisationExtendedModel(const G4Parti
 
 G4DNARuddIonisationExtendedModel::~G4DNARuddIonisationExtendedModel()
 {  
-  if(isFirst) {
+  if (isFirst) {
     for(auto & i : xsdata) { delete i; }
   }
 }
@@ -330,18 +330,23 @@ G4DNARuddIonisationExtendedModel::SampleSecondaries(std::vector<G4DynamicParticl
   }
   // check energy balance
   // remaining excitation energy of water molecule
-  G4double exc = bindingEnergy - esum;
+  G4double exc = std::max(bindingEnergy - esum, 0.0);
 
   // remaining projectile energy
   G4double scatteredEnergy = kinE - bindingEnergy - esec;
-  if(scatteredEnergy < -tolerance || exc < -tolerance) {
+  if (scatteredEnergy < -tolerance || exc < -tolerance) {
     G4cout << "G4DNARuddIonisationExtendedModel::SampleSecondaries: "
            << "negative final E(keV)=" << scatteredEnergy/CLHEP::keV << " Ein(keV)="
            << kinE/CLHEP::keV << "  " << pd->GetParticleName()
            << " Edelta(keV)=" << esec/CLHEP::keV << " MeV, Exc(keV)=" << exc/CLHEP::keV
 	   << G4endl;
   }
-
+  scatteredEnergy = std::max(scatteredEnergy, 0.0);
+  /*
+  G4cout << "Eprim(keV)=" << kinE/CLHEP::keV << " Efin(keV)=" << scatteredEnergy/CLHEP::keV
+	 << " Esec(keV)=" << esec/CLHEP::keV << " Exc(keV)=" << exc/CLHEP::keV
+	 << " tolerance(keV)=" << tolerance/CLHEP::keV << G4endl; 
+  */
   // projectile
   if (!statCode) {
     fParticleChangeForGamma->SetProposedKineticEnergy(scatteredEnergy);
@@ -447,8 +452,6 @@ G4double G4DNARuddIonisationExtendedModel::SampleElectronEnergy(G4double kine,
 
   // find max probability
   G4double pmax = ProbabilityFunction(kine, 0.0, shell);
-  //G4cout << "## E(keV)=" << kine/keV << " emax=" << emax/keV
-  //       << " pmax(0)=" << pmax << " shell=" << shell << " nn=" << nn << G4endl;
 
   G4double e0 = 0.0; // energy with max probability
   // 2 areas after point with max probability
