@@ -50,6 +50,7 @@
 #include "G4EmExtraParameters.hh"
 #include "G4EmLowEParameters.hh"
 #include "G4EmParametersMessenger.hh"
+#include "G4EmUtility.hh"
 #include "G4NistManager.hh"
 #include "G4RegionStore.hh"
 #include "G4Region.hh"
@@ -590,16 +591,12 @@ G4bool G4EmParameters::IsPrintLocked() const
 
 G4EmSaturation* G4EmParameters::GetEmSaturation()
 {
-  if(nullptr == emSaturation) { 
-#ifdef G4MULTITHREADED
-    G4MUTEXLOCK(&emParametersMutex);
-    if(nullptr == emSaturation) { 
-#endif
+  if (nullptr == emSaturation) {
+    G4AutoLock l(&emParametersMutex);
+    if (nullptr == emSaturation) { 
       emSaturation = new G4EmSaturation(1);
-#ifdef G4MULTITHREADED
     }
-    G4MUTEXUNLOCK(&emParametersMutex);
-#endif
+    l.unlock();
   }
   birks = true;
   return emSaturation;
@@ -719,7 +716,7 @@ G4double G4EmParameters::MaxEnergyFor5DMuPair() const
 void G4EmParameters::SetLinearLossLimit(G4double val)
 {
   if(IsLocked()) { return; }
-  if(val > 0.0 && val < 0.5) {
+  if(val > 0.0 && val < 1.0) {
     linLossLimit = val;
   } else {
     G4ExceptionDescription ed;
@@ -968,6 +965,28 @@ G4double G4EmParameters::ScreeningFactor() const
   return factorScreen;
 }
 
+void G4EmParameters::SetMaxDNAElectronEnergy(G4double val)
+{
+  if (IsLocked()) { return; }
+  fCParameters->SetMaxDNAElectronEnergy(val);
+}
+
+G4double G4EmParameters::MaxDNAElectronEnergy() const
+{
+  return fCParameters->MaxDNAElectronEnergy();
+}
+
+void G4EmParameters::SetMaxDNAIonEnergy(G4double val)
+{
+  if (IsLocked()) { return; }
+  fCParameters->SetMaxDNAIonEnergy(val);
+}
+
+G4double G4EmParameters::MaxDNAIonEnergy() const
+{
+  return fCParameters->MaxDNAIonEnergy();
+}
+
 void G4EmParameters::SetStepFunction(G4double v1, G4double v2)
 {
   if(IsLocked()) { return; }
@@ -1365,6 +1384,22 @@ const G4String& G4EmParameters::GetDirLEDATA() const
   return fDirLEDATA;
 }
 
+void G4EmParameters::SetFluctuationsForRegion(const G4String& nam, G4bool flag)
+{
+  if (IsLocked()) { return; }
+  G4String ss = fBParameters->CheckRegion(nam);
+  if (!fluctRegions.empty()) {
+    for (auto const& p : fluctRegions) { if (p.first == ss) { return; } }
+  }
+  fluctRegions.push_back(std::make_pair(ss, flag));
+}
+
+void G4EmParameters::DefineFluctuationFlags(std::vector<G4bool>* theFlags)
+{
+  if (fluctRegions.empty()) { return; }
+  G4EmUtility::FillFluctFlags(fluctRegions, theFlags);
+}
+
 void G4EmParameters::StreamInfo(std::ostream& os) const
 {
   G4long prec = os.precision(5);
@@ -1417,7 +1452,7 @@ void G4EmParameters::StreamInfo(std::ostream& os) const
   os << "5D gamma conversion limit for muon pair            " 
      << max5DEnergyForMuPair/CLHEP::GeV << " GeV\n";
   }
-  os << "Use Ricardo-Gerardo pair production model          "
+  os << "Use RiGe 5D e+e- pair production model by muons    "
      << fUseRiGePairProductionModel << "\n";
   os << "Livermore data directory                           " 
      << fCParameters->LivermoreDataDir() << "\n";
@@ -1442,12 +1477,30 @@ void G4EmParameters::StreamInfo(std::ostream& os) const
   os << "Lowest muon/hadron kinetic energy                  " 
      <<G4BestUnit(lowestMuHadEnergy,"Energy") << "\n";
   os << "Use ICRU90 data                                    " << fICRU90 << "\n";
-  os << "Fluctuations of dE/dx are enabled                  " <<lossFluctuation << "\n";
+  os << "Fluctuations of dE/dx are enabled                  " << lossFluctuation << "\n";
   G4String namef = "Universal";
   if(fFluct == fUrbanFluctuation) { namef = "Urban"; }
   else if(fFluct == fDummyFluctuation) { namef = "Dummy"; }
   os << "Type of fluctuation model for leptons and hadrons  " << namef << "\n";
-  os << "Use built-in Birks satuaration                     " << birks << "\n";
+  if (!fluctRegions.empty()) {
+    if (lossFluctuation) {
+      os << "Fluctuations of dE/dx are disabled in G4Regions:   ";
+    } else {
+      os << "Fluctuations of dE/dx are enabled in G4Regions:    ";
+    }
+    G4int n = 0;
+    for (auto const& p : fluctRegions) {
+      if (p.second != lossFluctuation) {
+        os << p.first << " ";
+	if (n > 0 && (n/2)*2 == n) {
+	  os << "\n" << "                                                   ";
+	}
+	++n;
+      }
+    }
+    os << "\n";
+  }
+  os << "Use built-in Birks saturation                      " << birks << "\n";
   os << "Build CSDA range enabled                           " <<buildCSDARange << "\n";
   os << "Use cut as a final range enabled                   " <<cutAsFinalRange << "\n";
   os << "Enable angular generator interface                 " 
@@ -1510,6 +1563,10 @@ void G4EmParameters::StreamInfo(std::ostream& os) const
   if(fDNA) {
   os << "======                 DNA Physics Parameters                  ========" << "\n";
   os << "=======================================================================" << "\n";
+  os << "Max DNA electron energy                            "
+     << fCParameters->MaxDNAElectronEnergy()/CLHEP::MeV << " MeV\n";    
+  os << "Max DNA ion energy                                 "
+     << fCParameters->MaxDNAIonEnergy()/CLHEP::MeV << " MeV\n";    
   os << "Use fast sampling in DNA models                    " 
      << fCParameters->DNAFast() << "\n";
   os << "Use Stationary option in DNA models                " 
