@@ -111,8 +111,7 @@ G4VEmProcess::G4VEmProcess(const G4String& name, G4ProcessType type):
 
 G4VEmProcess::~G4VEmProcess()
 {
-  if(isTheMaster) {
-    delete theData;
+  if (isTheMaster) {
     delete theEnergyOfCrossSectionMax;
   }
   delete modelManager;
@@ -192,15 +191,15 @@ void G4VEmProcess::PreparePhysicsTable(const G4ParticleDefinition& part)
 
   // prepare tables
   if(isTheMaster) {
-    if(nullptr == theData) { theData = new G4EmDataHandler(2); }
-
     if(buildLambdaTable) {
-      theLambdaTable = theData->MakeTable(0);
+      if (nullptr == theData) { theData = new G4EmDataHandler(2, particle->GetParticleName()); }
+      theLambdaTable = theData->MakeTable(theLambdaTable, 0);
       bld->InitialiseBaseMaterials(theLambdaTable);
     }
     // high energy table
     if(minKinEnergyPrim < maxKinEnergy) {
-      theLambdaTablePrim = theData->MakeTable(1);
+      if (nullptr == theData) { theData = new G4EmDataHandler(2, particle->GetParticleName()); }
+      theLambdaTablePrim = theData->MakeTable(theLambdaTablePrim, 1);
       bld->InitialiseBaseMaterials(theLambdaTablePrim);
     }
   }
@@ -353,6 +352,10 @@ void G4VEmProcess::StartTracking(G4Track* track)
       biasManager->ResetForcedInteraction(); 
     }
   }
+  for (G4int i=0; i<numberOfModels; ++i) {
+    auto ptr = GetModelByIndex(i);
+    ptr->StartTracking(track);
+  }
 }
 
 //....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
@@ -369,6 +372,9 @@ G4double G4VEmProcess::PostStepGetPhysicalInteractionLength(
   preStepKinEnergy = track.GetKineticEnergy();
   const G4double scaledEnergy = preStepKinEnergy*massRatio;
   SelectModel(scaledEnergy, currentCoupleIndex);
+
+  // In models applied to ions the dynamic charge is needed
+  if (isIon) { currentModel->ChargeSquareRatio(track); }
   /*
   G4cout << "PostStepGetPhysicalInteractionLength: idx= " << currentCoupleIndex
          << "  couple: " << currentCouple << G4endl;
