@@ -128,6 +128,7 @@ G4VEnergyLossProcess::G4VEnergyLossProcess(const G4String& name,
   G4LossTableBuilder* bld = lManager->GetTableBuilder();
   theDensityFactor = bld->GetDensityFactors();
   theDensityIdx = bld->GetCoupleIndexes();
+  theFluctuationFlags = bld->GetFluctuationFlags();
 
   scTracks.reserve(10);
   secParticles.reserve(12);
@@ -139,7 +140,6 @@ G4VEnergyLossProcess::G4VEnergyLossProcess(const G4String& name,
 G4VEnergyLossProcess::~G4VEnergyLossProcess()
 {
   if (isMaster) {
-    if(nullptr == baseParticle) { delete theData; }
     delete theEnergyOfCrossSectionMax;
     if(nullptr != fXSpeaks) {
       for(auto const & v : *fXSpeaks) { delete v; }
@@ -227,7 +227,7 @@ G4VEnergyLossProcess::PreparePhysicsTable(const G4ParticleDefinition& part)
   InitialiseEnergyLossProcess(particle, baseParticle);
 
   // parameters of the process
-  if(!actLossFluc) { lossFluctuationFlag = theParameters->LossFluctuation(); }
+  lossFluctuationFlag = theParameters->LossFluctuation();
   useCutAsFinalRange = theParameters->UseCutAsFinalRange();
   if(!actMinKinEnergy) { minKinEnergy = theParameters->MinKinEnergy(); }
   if(!actMaxKinEnergy) { maxKinEnergy = theParameters->MaxKinEnergy(); }
@@ -266,29 +266,26 @@ G4VEnergyLossProcess::PreparePhysicsTable(const G4ParticleDefinition& part)
 
   // Tables preparation
   if (isMaster && nullptr == baseParticle) {
-    if(nullptr == theData) { theData = new G4EmDataHandler(7); }
+    if (nullptr == theData) { theData = new G4EmDataHandler(7, particle->GetParticleName()); }
 
-    if(nullptr != theDEDXTable && isIonisation) {
-      if(nullptr != theIonisationTable && theDEDXTable != theIonisationTable) {
-	theData->CleanTable(0);
-	theDEDXTable = theIonisationTable;
-	theIonisationTable = nullptr;
-      }
+    if (isIonisation && theDEDXTable != theIonisationTable) {
+      theData->CleanTable(0);
+      theDEDXTable = theIonisationTable;
+      theIonisationTable = nullptr;
     }
-    
     theDEDXTable = theData->MakeTable(theDEDXTable, 0);
     bld->InitialiseBaseMaterials(theDEDXTable);
-    theData->UpdateTable(theIonisationTable, 1);
+    theData->MakeTable(theIonisationTable, 1);
 
     if (theParameters->BuildCSDARange()) {
-      theDEDXunRestrictedTable = theData->MakeTable(2);
-      if(isIonisation) { theCSDARangeTable = theData->MakeTable(3); }
+      theDEDXunRestrictedTable = theData->MakeTable(theDEDXunRestrictedTable, 2);
+      if(isIonisation) { theCSDARangeTable = theData->MakeTable(theCSDARangeTable, 3); }
     }
 
-    theLambdaTable = theData->MakeTable(4);
+    theLambdaTable = theData->MakeTable(theLambdaTable, 4);
     if(isIonisation) {
-      theRangeTableForLoss = theData->MakeTable(5);
-      theInverseRangeTable = theData->MakeTable(6);
+      theRangeTableForLoss = theData->MakeTable(theRangeTableForLoss, 5);
+      theInverseRangeTable = theData->MakeTable(theInverseRangeTable, 6);
     }
   }
 
@@ -310,13 +307,6 @@ G4VEnergyLossProcess::PreparePhysicsTable(const G4ParticleDefinition& part)
   if(isIonisation) { 
     subcutProducer = lManager->SubCutProducer();
   }
-  if(1 == nSCoffRegions) {
-    if((*scoffRegions)[0]->GetName() == "DefaultRegionForTheWorld") {
-      delete scoffRegions;
-      scoffRegions = nullptr;
-      nSCoffRegions = 0;
-    }
-  }
 
   if(1 < verboseLevel) {
     G4cout << "G4VEnergyLossProcess::PrepearPhysicsTable() is done "
@@ -552,7 +542,7 @@ void G4VEnergyLossProcess::StartTracking(G4Track* track)
 
   // reset ion
   if(isIon) {
-    const G4double newmass = track->GetDefinition()->GetPDGMass();
+    const G4double newmass = track->GetParticleDefinition()->GetPDGMass();
     massRatio = (nullptr == baseParticle) ? CLHEP::proton_mass_c2/newmass
       : baseParticle->GetPDGMass()/newmass;
     logMassRatio = G4Log(massRatio);
@@ -622,7 +612,7 @@ G4double G4VEnergyLossProcess::PostStepGetPhysicalInteractionLength(
     reduceFactor = 1.0/(fFactor*massRatio);
     if (lossFluctuationFlag) {
       auto fluc = currentModel->GetModelOfFluctuations();
-      fluc->SetParticleAndCharge(track.GetDefinition(), q2);
+      fluc->SetParticleAndCharge(track.GetParticleDefinition(), q2);
     }
   }
 
@@ -667,7 +657,7 @@ G4double G4VEnergyLossProcess::PostStepGetPhysicalInteractionLength(
   if (verboseLevel>2) {
     G4cout << "G4VEnergyLossProcess::PostStepGetPhysicalInteractionLength ";
     G4cout << "[ " << GetProcessName() << "]" << G4endl; 
-    G4cout << " for " << track.GetDefinition()->GetParticleName() 
+    G4cout << " for " << track.GetParticleDefinition()->GetParticleName() 
            << " in Material  " <<  currentMaterial->GetName()
            << " Ekin(MeV)= " << preStepKinEnergy/MeV 
            << " track material: " << track.GetMaterial()->GetName()
@@ -809,7 +799,7 @@ G4VParticleChange* G4VEnergyLossProcess::AlongStepDoIt(const G4Track& track,
     if (useDeexcitation) {
       atomDeexcitation->AlongStepDeexcitation(scTracks, step, 
                                               eloss, (G4int)currentCoupleIndex);
-      if(scTracks.size() > 0) { FillSecondariesAlongStep(weight); }
+      if (!scTracks.empty()) { FillSecondariesAlongStep(weight); }
       eloss = std::max(eloss, 0.0);
     }
     fParticleChange.SetProposedKineticEnergy(0.0);
@@ -861,8 +851,11 @@ G4VParticleChange* G4VEnergyLossProcess::AlongStepDoIt(const G4Track& track,
   G4double esec = 0.0;
 
   // Corrections, which cannot be tabulated
-  if(isIon) {
-    currentModel->CorrectionsAlongStep(currentCouple, dynParticle, 
+  if (isIon) {
+    currentModel->SetCurrentCouple(currentCouple);
+    currentModel->CorrectionsAlongStep(currentMaterial,
+				       track.GetParticleDefinition(),
+				       preStepKinEnergy, cut,
                                        length, eloss);
     eloss = std::max(eloss, 0.0);
   }
@@ -871,7 +864,7 @@ G4VParticleChange* G4VEnergyLossProcess::AlongStepDoIt(const G4Track& track,
   if(eloss >= preStepKinEnergy) {
     eloss = preStepKinEnergy;
 
-  } else if (lossFluctuationFlag) {
+  } else if ((*theFluctuationFlags)[currentCoupleIndex]) {
     const G4double tmax = currentModel->MaxSecondaryKinEnergy(dynParticle);
     const G4double tcut = std::min(cut, tmax);
     G4VEmFluctuationModel* fluc = currentModel->GetModelOfFluctuations();
@@ -963,7 +956,7 @@ void G4VEnergyLossProcess::FillSecondariesAlongStep(G4double wt)
     if(nullptr != t) {
       t->SetWeight(weight); 
       pParticleChange->AddSecondary(t);
-      G4int pdg = t->GetDefinition()->GetPDGEncoding();
+      G4int pdg = t->GetParticleDefinition()->GetPDGEncoding();
       if (i < n0) {
         if (pdg == 22) {
 	  t->SetCreatorModelID(gpixeID);
@@ -1423,7 +1416,15 @@ void G4VEnergyLossProcess::SetIonisation(G4bool val)
 
 //....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
 
- void G4VEnergyLossProcess::SetLinearLossLimit(G4double val)
+void G4VEnergyLossProcess::SetLossFluctuations(G4bool)
+{
+  G4cout << "### G4VEnergyLossProcess::SetLossFluctuations has no effect and "
+	 << "will be removed for the next major release" << G4endl;
+}
+
+//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
+
+void G4VEnergyLossProcess::SetLinearLossLimit(G4double val)
 {
   if(0.0 < val && val < 1.0) { 
     linLossLimit = val;
@@ -1435,11 +1436,11 @@ void G4VEnergyLossProcess::SetIonisation(G4bool val)
 
 void G4VEnergyLossProcess::SetStepFunction(G4double v1, G4double v2)
 {
-  if(0.0 < v1 && 0.0 < v2) { 
+  if(0.0 < v1 && 1.0 >= v1 && 0.0 < v2) { 
     dRoverRange = std::min(1.0, v1);
     finalRange = std::min(v2, 1.e+50);
   } else {
-    PrintWarning("SetStepFunctionV1", v1); 
+    PrintWarning("SetStepFunctionV1", v1);
     PrintWarning("SetStepFunctionV2", v2); 
   }
 }
