@@ -68,6 +68,8 @@
 #include "G4INCLEtaNToPiPiNChannel.hh"
 #include "G4INCLOmegaNElasticChannel.hh"
 #include "G4INCLOmegaNToPiNChannel.hh"
+#include "G4INCLEtaOrOmegaNToLKChannel.hh"
+#include "G4INCLEtaOrOmegaNToSKChannel.hh"
 #include "G4INCLNNToNLKChannel.hh"
 #include "G4INCLNNToNSKChannel.hh"
 #include "G4INCLNNToNLKpiChannel.hh"
@@ -141,6 +143,67 @@ namespace G4INCL {
 
   BinaryCollisionAvatar::~BinaryCollisionAvatar() {
   }
+ 
+void BinaryCollisionAvatar::generateSrcPairsMethod(ParticleList &theList,
+                                                   const G4int theN,
+                                                   const G4int theZ,
+                                                   G4INCL::Particle *p1,
+                                                   G4INCL::Particle *p2) {
+
+  std::vector<ThreeVector> posp;
+  std::vector<ThreeVector> posn;
+  posp.resize(theZ);
+  posn.resize(theN);
+
+  for (int i = 0; i < theZ + theN; ++i) {
+    theList[i]->setNumberOfSrcPair(0);
+  }
+  theNucleus->getStore()->getBook().setSrcPairs(0);
+
+  // Check that the loops in theZ and theN do what we wish!
+  G4int npairs = 0;
+  for (G4int i = 0; i < theZ; ++i) {
+    Particle *ap = theList[i];
+    posp[i] = ap->getPosition();
+    for (G4int j = 0; j < theN; ++j) {
+      Particle *an = theList[j + theZ];
+      posn[j] = an->getPosition();
+      if ((posn[j] - posp[i]).mag() < ParticleTable::getsrcPairDistance() &&
+          ap->getSrcPair() == 0 && an->getSrcPair() == 0 &&
+          ap->getType() != an->getType() && ap->isTargetSpectator() == 1 &&
+          an->isTargetSpectator() == 1) {
+        npairs++;
+        theNucleus->getStore()->getBook().incrementSrcPairs();
+        if (an == p1 || an == p2 || ap == p1 || ap == p2) {
+          theList[i]->setNumberOfSrcPair(npairs);
+          theList[j + theZ]->setNumberOfSrcPair(npairs);
+        }
+      }
+    }
+  }
+
+  G4int nbp = 0, nbn = 0;
+  for (G4int i = 0; i < theZ + theN; ++i) {
+
+    if (theList[i]->getSrcPair() > 0 && theList[i]->getType() == Proton)
+      nbp++;
+
+    if (theList[i]->getSrcPair() > 0 && theList[i]->getType() == Neutron)
+      nbn++;
+  }
+
+  if (nbp != nbn) {
+    INCL_DEBUG("Pairs: " << nbp << " " << nbn << '\n');
+    for (G4int i = 0; i < theZ; ++i) {
+      INCL_DEBUG(ParticleTable::getName(theList[i]->getType())
+                 << " " << theList[i]->isTargetSpectator() << " "
+                 << theList[i]->getSrcPair());
+    }
+    INCL_DEBUG("----------- End ------------------" << '\n');
+  }
+
+  return;
+}
 
   G4INCL::IChannel* BinaryCollisionAvatar::getChannel() {
     // We already check cutNN at avatar creation time, but we have to check it
@@ -182,12 +245,21 @@ namespace G4INCL {
     minimumDistance -= particle2->getPosition();
     const G4double betaDotX = boostVector.dot(minimumDistance);
     const G4double minDist = Math::tenPi*(minimumDistance.mag2() + betaDotX*betaDotX / (1.-boostVector.mag2()));
-    if(minDist > theCrossSection) {
+
+    Config const *theConfig=theNucleus->getStore()->getConfig();
+    if ((minDist > theCrossSection) && 
+        (!((particle1->getType()==antiProton  && particle1->getKineticEnergy() <= theConfig->getAtrestThreshold())   ||
+           (particle2->getType()==antiProton  && particle2->getKineticEnergy() <= theConfig->getAtrestThreshold())   ||
+           (particle1->getType()==antiNeutron && particle1->getKineticEnergy() <= particle1->getPotentialEnergy()) ||
+           (particle2->getType()==antiNeutron && particle2->getKineticEnergy() <= particle2->getPotentialEnergy())))) {
+      if(!((particle1->isAntiNucleon() && particle1->getEnergy() <= particle1->getINCLMass()) ||
+        (particle2->isAntiNucleon() && particle2->getEnergy() <= particle2->getINCLMass()))){
       INCL_DEBUG("CM distance of approach is too small: " << minDist << ">" <<
         theCrossSection <<"; returning a NULL channel" << '\n');
       InteractionAvatar::restoreParticles();
       return NULL;
     }
+    }
 
     /** Bias apply for this reaction in order to get the same
      * ParticleBias for all stange particles.
@@ -261,15 +333,50 @@ namespace G4INCL {
       if(elasticCX > rChannel) {
 // Elastic NN channel
         isElastic = true;
+      weight = counterweight;
+      if (theNucleus->getStore()->getBook().getAcceptedCollisions() == 0 &&
+          theNucleus->getStore()->getBook().getAcceptedSrcCollisions() == 0 &&
+          ParticleTable::getsrcPairConfig()) {
+        INCL_DEBUG("NN-SRC interaction: elastic channel chosen" << '\n');
+        ParticleList &inside = theNucleus->getStore()->getParticlesforSrc();
+        G4int zz = theNucleus->getZ();
+        generateSrcPairsMethod(inside, theNucleus->getA() - zz, zz, particle1,
+                               particle2);
+        if ((particle1->getSrcPair() > 0 || particle2->getSrcPair() > 0)) {
+          return new ElasticChannel(particle1, particle2, theNucleus);
+        } else {
+          INCL_DEBUG("NN interaction: elastic channel chosen" << '\n');
+          theNucleus->resetSrc();
+          return new ElasticChannel(particle1, particle2);
+        }
+      } else {
         INCL_DEBUG("NN interaction: elastic channel chosen" << '\n');
-        weight = counterweight;
         return new ElasticChannel(particle1, particle2);
+        }
       } else if((elasticCX + deltaProductionCX) > rChannel) {
         isElastic = false;
 // NN -> N Delta channel is chosen
+      weight = counterweight;
+      if (theNucleus->getStore()->getBook().getAcceptedCollisions() == 0 &&
+          theNucleus->getStore()->getBook().getAcceptedSrcCollisions() == 0 &&
+          ParticleTable::getsrcPairConfig()) {
+        INCL_DEBUG("NN-SRC interaction: Delta channel chosen" << '\n');
+        ParticleList &inside = theNucleus->getStore()->getParticlesforSrc();
+        G4int zz = theNucleus->getZ();
+        generateSrcPairsMethod(inside, theNucleus->getA() - zz, zz, particle1,
+                               particle2);
+
+        if ((particle1->getSrcPair() > 0 || particle2->getSrcPair() > 0)) {
+          return new DeltaProductionChannel(particle1, particle2, theNucleus);
+        } else {
+          INCL_DEBUG("NN interaction: Delta channel chosen" << '\n');
+          theNucleus->resetSrc();
+          return new DeltaProductionChannel(particle1, particle2);
+        }
+      } else {
         INCL_DEBUG("NN interaction: Delta channel chosen" << '\n');
-        weight = counterweight;
         return new DeltaProductionChannel(particle1, particle2);
+        }
       } else if(elasticCX + deltaProductionCX + onePiProductionCX > rChannel) {
         isElastic = false;
 // NN -> PiNN channel is chosen
@@ -948,8 +1055,10 @@ namespace G4INCL {
                     const G4double elasticCX = CrossSections::elastic(particle1, particle2);
                     const G4double onePiProductionCX = CrossSections::etaNToPiN(particle1, particle2);
                     const G4double twoPiProductionCX = CrossSections::etaNToPiPiN(particle1, particle2);
+                    const G4double LKProductionCX = CrossSections::etaNToLK(particle1, particle2);
+                    const G4double SKProductionCX = CrossSections::etaNToSK(particle1, particle2);
                     const G4double totCX=CrossSections::total(particle1, particle2);
-// assert(std::fabs(totCX-elasticCX-onePiProductionCX-twoPiProductionCX)<1.);
+// assert(std::fabs(totCX-elasticCX-onePiProductionCX-twoPiProductionCX)-LKProductionCX-SKProductionCX<1.);
                     
                     const G4double rChannel=Random::shoot() * totCX;
                                         
@@ -968,11 +1077,29 @@ namespace G4INCL {
 // EtaN -> EtaPiPiN channel is chosen
                         INCL_DEBUG("EtaN interaction: PiPiN channel chosen" << '\n');
                         return new EtaNToPiPiNChannel(particle1, particle2);
+                    } else if(elasticCX + onePiProductionCX + twoPiProductionCX + LKProductionCX > rChannel) {
+                        isElastic = false;
+// EtaN -> LK channel is chosen
+                        INCL_DEBUG("EtaN interaction: LK channel chosen" << '\n');
+                        return new EtaOrOmegaNToLKChannel(particle1, particle2);
+                    } else if(elasticCX + onePiProductionCX + twoPiProductionCX + LKProductionCX + SKProductionCX > rChannel) {
+                        isElastic = false;
+// EtaN -> SK channel is chosen
+                        INCL_DEBUG("EtaN interaction: SK channel chosen" << '\n');
+                        return new EtaOrOmegaNToSKChannel(particle1, particle2);
                     }
 
                     else {
                         INCL_WARN("inconsistency within the EtaN Cross Sections (sum!=inelastic)" << '\n');
-                        if(twoPiProductionCX>0.) {
+                        if(SKProductionCX>0.) {
+                            INCL_WARN("Returning a SK channel" << '\n');
+                            isElastic = false;
+                            return new EtaOrOmegaNToSKChannel(particle1, particle2);
+                        } else if(LKProductionCX>0.) {
+                            INCL_WARN("Returning a LK channel" << '\n');
+                            isElastic = false;
+                            return new EtaOrOmegaNToLKChannel(particle1, particle2);
+                        } else if(twoPiProductionCX>0.) {
                             INCL_WARN("Returning a PiPiN channel" << '\n');
                             isElastic = false;
                             return new EtaNToPiPiNChannel(particle1, particle2);
@@ -993,8 +1120,10 @@ namespace G4INCL {
                     const G4double elasticCX = CrossSections::elastic(particle1, particle2);
                     const G4double onePiProductionCX = CrossSections::omegaNToPiN(particle1, particle2);
                     const G4double twoPiProductionCX = CrossSections::omegaNToPiPiN(particle1, particle2);
+                    const G4double LKProductionCX = CrossSections::omegaNToLK(particle1, particle2);
+                    const G4double SKProductionCX = CrossSections::omegaNToSK(particle1, particle2);
                     const G4double totCX=CrossSections::total(particle1, particle2);
-// assert(std::fabs(totCX-elasticCX-onePiProductionCX-twoPiProductionCX)<1.);
+// assert(std::fabs(totCX-elasticCX-onePiProductionCX-twoPiProductionCX + LKProductionCX + SKProductionCX)<1.);
                     
                     const G4double rChannel=Random::shoot() * totCX;
      
@@ -1013,10 +1142,28 @@ namespace G4INCL {
 // OmegaN -> PiPiN channel is chosen
                         INCL_DEBUG("OmegaN interaction: PiPiN channel chosen" << '\n');
                         return new OmegaNToPiPiNChannel(particle1, particle2);
+                    } else if(elasticCX + onePiProductionCX + twoPiProductionCX + LKProductionCX > rChannel) {
+                        isElastic = false;
+// OmegaN -> LK channel is chosen
+                        INCL_DEBUG("EtaN interaction: LK channel chosen" << '\n');
+                        return new EtaOrOmegaNToLKChannel(particle1, particle2);
+                    } else if(elasticCX + onePiProductionCX + twoPiProductionCX + LKProductionCX + SKProductionCX > rChannel) {
+                        isElastic = false;
+// OmegaN -> SK channel is chosen
+                        INCL_DEBUG("EtaN interaction: SK channel chosen" << '\n');
+                        return new EtaOrOmegaNToSKChannel(particle1, particle2);
                     }
                     else {
                         INCL_WARN("inconsistency within the OmegaN Cross Sections (sum!=inelastic)" << '\n');
-                        if(twoPiProductionCX>0.) {
+                        if(SKProductionCX>0.) {
+                            INCL_WARN("Returning a SK channel" << '\n');
+                            isElastic = false;
+                            return new EtaOrOmegaNToSKChannel(particle1, particle2);
+                        } else if(LKProductionCX>0.) {
+                            INCL_WARN("Returning a LK channel" << '\n');
+                            isElastic = false;
+                            return new EtaOrOmegaNToLKChannel(particle1, particle2);
+                        } else if(twoPiProductionCX>0.) {
                             INCL_WARN("Returning a PiPiN channel" << '\n');
                             isElastic = false;
                             return new OmegaNToPiPiNChannel(particle1, particle2);
@@ -1242,6 +1389,66 @@ namespace G4INCL {
         }
     } else if ((particle1->isNucleon() && particle2->isAntiNucleon()) || (particle2->isNucleon() && particle1->isAntiNucleon())) {
 //// NNbar
+        
+        //Forcing annihilationfor emitInsideAntinucleon at the end of cascade &&  annihilation if E <= M + p_threhsold (from antideuteron in generateBinaryCollisionAvatar()) 
+        /*const Particle *antinucleon;
+        const Particle *nucleon;
+        
+        if (particle1->isAntiNucleon()) {
+            antinucleon = particle1;
+            nucleon = particle2;
+        }
+        else {
+            antinucleon = particle2;
+            nucleon = particle1;
+        }
+        double Esquared = antinucleon->getEnergy() * antinucleon->getEnergy();
+        double antinucleon_threshold=0;
+        if(antinucleon->getType()==antiNeutron)
+          antinucleon_threshold = theNucleus->getStore()->getConfig()->getnbAtrestThreshold();
+        else if(antinucleon->getType() == antiProton)
+          antinucleon_threshold = theNucleus->getStore()->getConfig()->getAtrestThreshold();
+        else
+          INCL_ERROR("neither antiproton nor antineutron");*/
+        //double Sum_at_rest = antinucleon->getINCLMass() * antinucleon->getINCLMass() + antinucleon_threshold*antinucleon_threshold;
+      // Force the annihilation when T < Threshold (at rest)
+        //Config const *theConfig=theNucleus->getStore()->getConfig();
+
+        if ((theCrossSection == 9999.) || // XS=9999. means force annihilation
+            ((particle1->getType()==antiProton  && particle1->getKineticEnergy() <= theConfig->getAtrestThreshold())   ||
+             (particle2->getType()==antiProton  && particle2->getKineticEnergy() <= theConfig->getAtrestThreshold())   ||
+             (particle1->getType()==antiNeutron && particle1->getKineticEnergy() <= particle1->getPotentialEnergy()) ||
+             (particle2->getType()==antiNeutron && particle2->getKineticEnergy() <= particle2->getPotentialEnergy()))  ||
+            ((particle1->getType()==antiProton  && particle1->getEnergy() <= particle1->getINCLMass())   ||
+             (particle2->getType()==antiProton  && particle2->getEnergy() <= particle2->getINCLMass())   ||
+             (particle1->getType()==antiNeutron && particle1->getEnergy() <= particle1->getINCLMass()) ||
+             (particle2->getType()==antiNeutron && particle2->getEnergy() <= particle2->getINCLMass())))
+        {
+            isElastic = false; 
+            AnnihilationType atype0;
+            if((particle1->getType()==antiProton && particle2->getType()==Proton) || (particle2->getType()==antiProton && particle1->getType()==Proton)){
+              atype0 = PTypeInFlight;
+            }
+            else if((particle1->getType()==antiProton && particle2->getType()==Neutron) || (particle2->getType()==antiProton && particle1->getType()==Neutron)){
+              atype0 = NTypeInFlight;
+            }
+            else if((particle1->getType()==antiNeutron && particle2->getType()==Proton) || (particle2->getType()==antiNeutron && particle1->getType()==Proton)){
+              atype0 = NbarPTypeInFlight;
+            }
+            else if((particle1->getType()==antiNeutron && particle2->getType()==Neutron) || (particle2->getType()==antiNeutron && particle1->getType()==Neutron)){
+              atype0 = NbarNTypeInFlight;
+            }
+            else{
+              atype0 = Def;
+              INCL_ERROR("Annihilation type problem " << '\n');
+            }
+            theNucleus->setAType(atype0);
+            return new NNbarToAnnihilationChannel(theNucleus, particle1, particle2);
+        }
+        //delete antinucleon;
+        //delete nucleon;  
+
+        // Usual interactions
         const G4double totCX = CrossSections::total(particle1, particle2);
         const G4double NNbElasticCX = CrossSections::NNbarElastic(particle1,particle2);
         const G4double NNbCEXCX = CrossSections::NNbarCEX(particle1,particle2);
