@@ -1,5 +1,9 @@
 /*
 # <<BEGIN-copyright>>
+# Copyright 2019, Lawrence Livermore National Security, LLC.
+# This file is part of the gidiplus package (https://github.com/LLNL/gidiplus).
+# gidiplus is licensed under the MIT license (see https://opensource.org/licenses/MIT).
+# SPDX-License-Identifier: MIT
 # <<END-copyright>>
 */
 
@@ -11,7 +15,6 @@
 
 #if defined __cplusplus
     extern "C" {
-    namespace GIDI {
 #endif
 
 #define nf_GnG_adaptiveQuadrature_MaxMaxDepth 20
@@ -24,7 +27,6 @@ nfu_status nf_GnG_adaptiveQuadrature( nf_GnG_adaptiveQuadrature_callback quadrat
 
 #if defined __cplusplus
     }
-    }
 #endif
 
 #endif          /* End of nf_integration_h_included. */
