@@ -134,6 +134,7 @@ void G4OpenGLViewer::InitializeGLView ()
   glClearDepth (1.0);
   glDisable (GL_LINE_SMOOTH);
   glDisable (GL_POLYGON_SMOOTH);
+  glDisable (GL_POINT_SMOOTH);
 
 // clear the buffers and window?
   ClearView ();
@@ -1031,7 +1032,7 @@ std::string G4OpenGLViewer::getRealPrintFilename() {
   return temp;
 }
 
-GLdouble G4OpenGLViewer::getSceneNearWidth()
+G4double G4OpenGLViewer::GetSceneNearWidth()
 {
   if (!fSceneHandler.GetScene()) {
     return 0;
