@@ -354,9 +354,8 @@ void G4VSceneHandler::AddCompound (const G4THitsMap<G4double>& hits) {
       if (mesh && mesh->IsActive()) {
 	MeshScoreMap scoreMap = mesh->GetScoreMap();
         const G4String& mapNam = const_cast<G4THitsMap<G4double>&>(hits).GetName();
-	for(MeshScoreMap::const_iterator i = scoreMap.cbegin();
-	    i != scoreMap.cend(); ++i) {
-	  const G4String& scoreMapName = i->first;
+	for(const auto& i : scoreMap) {
+	  const G4String& scoreMapName = i.first;
 	  if (scoreMapName == mapNam) {
 	    G4DefaultLinearColorMap colorMap("G4VSceneHandlerColorMap");
 	    scoreMapHits = true;
@@ -396,10 +395,9 @@ void G4VSceneHandler::AddCompound (const G4THitsMap<G4StatDouble>& hits) {
       G4VScoringMesh* mesh = scoringManager->GetMesh((G4int)iMesh);
       if (mesh && mesh->IsActive()) {
 	MeshScoreMap scoreMap = mesh->GetScoreMap();
-	for(MeshScoreMap::const_iterator i = scoreMap.cbegin();
-	    i != scoreMap.cend(); ++i) {
-	  const G4String& scoreMapName = i->first;
-	  const G4THitsMap<G4StatDouble>* foundHits = i->second;
+	for(const auto& i : scoreMap) {
+	  const G4String& scoreMapName = i.first;
+	  const G4THitsMap<G4StatDouble>* foundHits = i.second;
 	  if (foundHits == &hits) {
 	    G4DefaultLinearColorMap colorMap("G4VSceneHandlerColorMap");
 	    scoreMapHits = true;
@@ -472,8 +470,8 @@ void G4VSceneHandler::AddPrimitive (const G4Polymarker& polymarker) {
       G4Circle dot (polymarker);
       dot.SetWorldSize  (0.);
       dot.SetScreenSize (0.1);  // Very small circle.
-      for (std::size_t iPoint = 0; iPoint < polymarker.size (); ++iPoint) {
-        dot.SetPosition (polymarker[iPoint]);
+      for (const auto& iPoint : polymarker) {
+        dot.SetPosition (iPoint);
         AddPrimitive (dot);
       }
     }
@@ -481,8 +479,8 @@ void G4VSceneHandler::AddPrimitive (const G4Polymarker& polymarker) {
     case G4Polymarker::circles:
     {
       G4Circle circle (polymarker);  // Default circle
-      for (std::size_t iPoint = 0; iPoint < polymarker.size (); ++iPoint) {
-        circle.SetPosition (polymarker[iPoint]);
+      for (const auto& iPoint : polymarker) {
+        circle.SetPosition (iPoint);
         AddPrimitive (circle);
       }
     }
@@ -490,8 +488,8 @@ void G4VSceneHandler::AddPrimitive (const G4Polymarker& polymarker) {
     case G4Polymarker::squares:
     {
       G4Square square (polymarker);  // Default square
-      for (std::size_t iPoint = 0; iPoint < polymarker.size (); ++iPoint) {
-        square.SetPosition (polymarker[iPoint]);
+      for (const auto& iPoint : polymarker) {
+        square.SetPosition (iPoint);
         AddPrimitive (square);
       }
     }
@@ -639,7 +637,6 @@ void G4VSceneHandler::RequestPrimitives (const G4VSolid& solid)
       // entry, something we would want to avoid.
       dots.SetVisAttributes(fpVisAttribs);
       dots.SetMarkerType(G4Polymarker::dots);
-      dots.SetSize(G4VMarker::screen,1.);
       G4int numberOfCloudPoints = GetNumberOfCloudPoints(fpVisAttribs);
       if (numberOfCloudPoints <= 0) numberOfCloudPoints = vp.GetNumberOfCloudPoints();
       for (G4int i = 0; i < numberOfCloudPoints; ++i) {
@@ -682,10 +679,10 @@ void G4VSceneHandler::ProcessScene()
     return;
 
   if(fpScene->GetExtent() == G4VisExtent::GetNullExtent())
-  {
+    {
     G4Exception("G4VSceneHandler::ProcessScene", "visman0106", JustWarning,
                 "The scene has no extent.");
-  }
+    }
 
   G4VisManager* visManager = G4VisManager::GetInstance();
 
@@ -696,16 +693,10 @@ void G4VSceneHandler::ProcessScene()
 
   fReadyForTransients = false;
 
-  // Reset fMarkForClearingTransientStore. (Leaving
-  // fMarkForClearingTransientStore true causes problems with
-  // recomputing transients below.)  Restore it again at end...
-  G4bool tmpMarkForClearingTransientStore = fMarkForClearingTransientStore;
-  fMarkForClearingTransientStore          = false;
-
   // Traverse geometry tree and send drawing primitives to window(s).
 
   const std::vector<G4Scene::Model>& runDurationModelList =
-    fpScene->GetRunDurationModelList();
+  fpScene->GetRunDurationModelList();
 
   if(runDurationModelList.size()) {
     if(verbosity >= G4VisManager::confirmations) {
@@ -728,9 +719,9 @@ void G4VSceneHandler::ProcessScene()
     // Create modeling parameters from view parameters...
     G4ModelingParameters* pMP = CreateModelingParameters();
 
-    for(std::size_t i = 0; i < runDurationModelList.size(); ++i) {
-      if(runDurationModelList[i].fActive) {
-        fpModel = runDurationModelList[i].fpModel;
+    for(const auto& i : runDurationModelList) {
+      if(i.fActive) {
+        fpModel = i.fpModel;
         fpModel->SetModelingParameters(pMP);
 
         // Describe to the current scene handler
@@ -747,6 +738,9 @@ void G4VSceneHandler::ProcessScene()
         if (pPVModel) {
           G4VViewer::SceneTreeScene sceneTreeScene(fpViewer, pPVModel);
           fpModel->DescribeYourselfTo(sceneTreeScene);
+          const auto& maxDepth = pPVModel->GetMaxFullDepth();
+          if (fMaxGeometryDepth < maxDepth) {fMaxGeometryDepth = maxDepth;}
+          // This will be the maximum geometry depth for this scene handler
         }
 
         // Reset modeling parameters pointer
@@ -761,23 +755,32 @@ void G4VSceneHandler::ProcessScene()
   }
 
   // Some printing
-  if(verbosity >= G4VisManager::confirmations) {
+  if(verbosity >= G4VisManager::parameters) {
     for (const auto& model: runDurationModelList) {
       if (model.fActive) {
         auto pvModel = dynamic_cast<G4PhysicalVolumeModel*>(model.fpModel);
         if (pvModel) {
-          G4int nTouchables = 0;
-          G4cout << "Numbers of touchables by depth in model \""
+
+          G4cout << "Numbers of all touchables by depth \""
           << pvModel->GetGlobalDescription() << "\":";
-          for (const auto& dn : pvModel->GetNumberOfTouchables()) {
+          for (const auto& dn : pvModel->GetMapOfAllTouchables()) {
             G4cout << "\n  Depth " << dn.first << ": " << dn.second;
-            nTouchables += dn.second;
           }
-          G4cout << "\n  Total number of touchables: " << nTouchables << G4endl;
+          G4cout << "\n  Total number of all touchables: "
+          << pvModel->GetTotalAllTouchables() << G4endl;
+
+          G4cout << "Numbers of touchables drawn by depth \""
+          << pvModel->GetGlobalDescription() << "\":";
+          for (const auto& dn : pvModel->GetMapOfDrawnTouchables()) {
+            G4cout << "\n  Depth " << dn.first << ": " << dn.second;
+          }
+          G4cout << "\n  Total number of drawn touchables: "
+          << pvModel->GetTotalDrawnTouchables() << G4endl;
         }
       }
     }
-
+  }
+  if(verbosity >= G4VisManager::warnings) {
     if (fProblematicVolumes.size() > 0) {
       G4cout << "Problematic volumes:";
       for (const auto& prob: fProblematicVolumes) {
@@ -787,6 +790,22 @@ void G4VSceneHandler::ProcessScene()
     }
   }
 
+  ProcessTransients();
+}
+
+void G4VSceneHandler::ProcessTransients()
+{
+  // Assumes transient store has already been cleared
+
+  // Reset fMarkForClearingTransientStore. (Leaving
+  // fMarkForClearingTransientStore true causes problems with
+  // recomputing transients below.)  Restore it again at end...
+  G4bool tmpMarkForClearingTransientStore = fMarkForClearingTransientStore;
+  fMarkForClearingTransientStore          = false;
+
+  G4VisManager* visManager = G4VisManager::GetInstance();
+  G4VisManager::Verbosity verbosity = visManager->GetVerbosity();
+
   fReadyForTransients = true;
 
   // Refresh event from end-of-event model list.
@@ -794,75 +813,75 @@ void G4VSceneHandler::ProcessScene()
   G4StateManager* stateManager = G4StateManager::GetStateManager();
   G4ApplicationState state     = stateManager->GetCurrentState();
   if(state == G4State_Idle || state == G4State_GeomClosed)
-  {
+    {
     visManager->SetEventRefreshing(true);
 
     if(visManager->GetRequestedEvent())
-    {
+      {
       DrawEvent(visManager->GetRequestedEvent());
-    }
+      }
     else
-    {
+      {
       G4RunManager* runManager = G4RunManagerFactory::GetMasterRunManager();
       if(runManager)
-      {
+        {
         const G4Run* run = runManager->GetCurrentRun();
         // Draw a null event in order to pick up models for the scene tree even before a run
         if (run == nullptr) DrawEvent(0);
         const std::vector<const G4Event*>* events =
-          run ? run->GetEventVector() : 0;
+        run ? run->GetEventVector() : 0;
         std::size_t nKeptEvents = 0;
         if(events)
           nKeptEvents = events->size();
         if(nKeptEvents)
-        {
-          if(fpScene->GetRefreshAtEndOfEvent())
           {
-            if(verbosity >= G4VisManager::confirmations)
+          if(fpScene->GetRefreshAtEndOfEvent())
             {
+            if(verbosity >= G4VisManager::confirmations)
+              {
               G4cout << "Refreshing event..." << G4endl;
-            }
+              }
             const G4Event* event = 0;
             if(events && events->size())
               event = events->back();
             if(event)
               DrawEvent(event);
-          }
+            }
           else
-          {  // Accumulating events.
+            {  // Accumulating events.
 
-            if(verbosity >= G4VisManager::confirmations)
-            {
-              G4cout << "Refreshing events in run..." << G4endl;
-            }
-            for(const auto& event : *events)
-            {
-              if(event)
-                DrawEvent(event);
-            }
+              if(verbosity >= G4VisManager::confirmations)
+                {
+                G4cout << "Refreshing events in run..." << G4endl;
+                }
+              for(const auto& event : *events)
+                {
+                if(event)
+                  DrawEvent(event);
+                }
 
-            if(!fpScene->GetRefreshAtEndOfRun())
-            {
-              if(verbosity >= G4VisManager::warnings)
-              {
-                G4warn << "WARNING: Cannot refresh events accumulated over more"
-                          "\n  than one runs.  Refreshed just the last run."
-                       << G4endl;
-              }
+              if(!fpScene->GetRefreshAtEndOfRun())
+                {
+                if(verbosity >= G4VisManager::warnings)
+                  {
+                  G4warn << "WARNING: Cannot refresh events accumulated over more"
+                  "\n  than one runs.  Refreshed just the last run."
+                  << G4endl;
+                  }
+                }
             }
           }
         }
       }
-    }
     visManager->SetEventRefreshing(false);
-  }
+    }
 
   // Refresh end-of-run model list.
   // Allow only in Idle or GeomClosed state...
   if(state == G4State_Idle || state == G4State_GeomClosed)
-  {
+    {
     DrawEndOfRunModels();
-  }
+    }
 
   fMarkForClearingTransientStore = tmpMarkForClearingTransientStore;
 }
@@ -995,9 +1014,14 @@ G4ModelingParameters* G4VSceneHandler::CreateModelingParameters ()
   
   pModelingParams->SetVisAttributesModifiers(vp.GetVisAttributesModifiers());
 
+  pModelingParams->SetTimeParameters(vp.GetTimeParameters());
+
   pModelingParams->SetSpecialMeshRendering(vp.IsSpecialMeshRendering());
   pModelingParams->SetSpecialMeshVolumes(vp.GetSpecialMeshVolumes());
 
+  pModelingParams->SetTransparencyByDepth(vp.GetTransparencyByDepth());
+  pModelingParams->SetTransparencyByDepthOption(vp.GetTransparencyByDepthOption());
+
   return pModelingParams;
 }
 
@@ -1264,6 +1288,17 @@ G4double G4VSceneHandler::GetMarkerSize
 (const G4VMarker& marker, 
  G4VSceneHandler::MarkerSizeType& markerSizeType)
 {
+  // Deal with G4Polymarker::dots
+  const auto& vp = fpViewer->GetViewParameters();
+  try {
+    const auto& polymarker = dynamic_cast<const G4Polymarker&>(marker);
+    if (polymarker.GetMarkerType() == G4Polymarker::dots) {
+      return vp.GetDotsSize();
+    }
+  }
+  catch (const std::bad_cast&) {}  // Continue
+
+  // Other markers
   G4bool userSpecified = marker.GetWorldSize() || marker.GetScreenSize();
   const G4VMarker& defaultMarker =
     fpViewer -> GetViewParameters().GetDefaultMarker();
@@ -1308,8 +1343,8 @@ std::ostream& operator << (std::ostream& os, const G4VSceneHandler& sh) {
 
   os << "Scene handler " << sh.fName << " has "
      << sh.fViewerList.size () << " viewer(s):";
-  for (std::size_t i = 0; i < sh.fViewerList.size (); ++i) {
-    os << "\n  " << *(sh.fViewerList [i]);
+  for (const auto* i : sh.fViewerList) {
+    os << "\n  " << *i;
   }
 
   if (sh.fpScene) {
@@ -1508,7 +1543,6 @@ void G4VSceneHandler::Draw3DRectMeshAsDots(const G4Mesh& mesh)
       dots.SetInfo(name);
       dots.SetVisAttributes(visAtts);
       dots.SetMarkerType(G4Polymarker::dots);
-      dots.SetSize(G4VMarker::screen,1.);
       // Enter empty polymarker into the map
       dotsByMaterial[material] = dots;
       // Now fill it in situ
@@ -1790,7 +1824,6 @@ void G4VSceneHandler::DrawTetMeshAsDots(const G4Mesh& mesh)
       G4Polymarker dots;
       dots.SetVisAttributes(visAtts);
       dots.SetMarkerType(G4Polymarker::dots);
-      dots.SetSize(G4VMarker::screen,1.);
       dots.SetInfo(name);
       // Enter empty polymarker into the map
       dotsByMaterial[material] = dots;
