@@ -96,8 +96,7 @@ G4VisCommandsTouchable::G4VisCommandsTouchable()
    "\nFor example, \"/Shap/\" matches \"Shape1\" and \"Shape2\".");
   fpCommandFindPath -> SetGuidance
   ("It may help to see a textual representation of the geometry hierarchy of"
-   "\nthe worlds. Try \"/vis/drawTree [worlds]\" or one of the driver/browser"
-   "\ncombinations that have the required functionality, e.g., HepRep.");
+   "\nthe worlds. Try \"/vis/drawTree [worlds]\".");
   G4UIparameter* parameter;
   parameter = new G4UIparameter ("physical-volume-name", 's', omitable = true);
   parameter -> SetDefaultValue ("world");
@@ -314,13 +313,15 @@ void G4VisCommandsTouchable::SetNewValue
       const G4Point3D& standardTargetPoint = currentScene->GetStandardTargetPoint();
       newVP.SetCurrentTargetPoint(newTargetPoint - standardTargetPoint);
 
-      // Interpolate
-      auto keepVisVerbose = fpVisManager->GetVerbosity();
-      fpVisManager->SetVerboseLevel(G4VisManager::errors);
-      if (newVP != saveVP) InterpolateToNewView(currentViewer, saveVP, newVP);
-      // ...and twinkle
-      Twinkle(currentViewer,newVP,touchables);
-      fpVisManager->SetVerboseLevel(keepVisVerbose);
+      if (currentViewer->GetKernelVisitElapsedTimeSeconds() < 0.1) {
+        // Interpolate
+        auto keepVisVerbose = fpVisManager->GetVerbosity();
+        fpVisManager->SetVerboseLevel(G4VisManager::errors);
+        if (newVP != saveVP) InterpolateToNewView(currentViewer, saveVP, newVP);
+        // ...and twinkle
+        Twinkle(currentViewer,newVP,touchables);
+        fpVisManager->SetVerboseLevel(keepVisVerbose);
+      }
 
       if (verbosity >= G4VisManager::confirmations) {
         G4cout
@@ -446,19 +447,23 @@ void G4VisCommandsTouchable::SetNewValue
 
   } else if (command == fpCommandTwinkle) {
 
-    G4PhysicalVolumeModel::TouchableProperties properties =
-    G4TouchableUtils::FindTouchableProperties(fCurrentTouchableProperties.fTouchablePath);
-    if (properties.fpTouchablePV) {
-      std::vector<std::vector<G4PhysicalVolumeModel::G4PhysicalVolumeNodeID>> touchables;
-      touchables.push_back(properties.fTouchableFullPVPath);
-      auto keepVisVerbose = fpVisManager->GetVerbosity();
-      fpVisManager->SetVerboseLevel(G4VisManager::errors);
-      auto keepVP = currentViewer->GetViewParameters();
-      Twinkle(currentViewer,currentViewer->GetViewParameters(),touchables);
-      SetViewParameters(currentViewer, keepVP);
-      fpVisManager->SetVerboseLevel(keepVisVerbose);
+    if (currentViewer->GetKernelVisitElapsedTimeSeconds() < 0.1) {
+      G4PhysicalVolumeModel::TouchableProperties properties =
+      G4TouchableUtils::FindTouchableProperties(fCurrentTouchableProperties.fTouchablePath);
+      if (properties.fpTouchablePV) {
+        std::vector<std::vector<G4PhysicalVolumeModel::G4PhysicalVolumeNodeID>> touchables;
+        touchables.push_back(properties.fTouchableFullPVPath);
+        auto keepVisVerbose = fpVisManager->GetVerbosity();
+        fpVisManager->SetVerboseLevel(G4VisManager::errors);
+        auto keepVP = currentViewer->GetViewParameters();
+        Twinkle(currentViewer,currentViewer->GetViewParameters(),touchables);
+        SetViewParameters(currentViewer, keepVP);
+        fpVisManager->SetVerboseLevel(keepVisVerbose);
+      } else {
+        G4warn << "Touchable not found." << G4endl;
+      }
     } else {
-      G4warn << "Touchable not found." << G4endl;
+      G4warn << "Twinkling not available - image construction time too long." << G4endl;
     }
     return;
 
